# Copyright (c) 2021 Felix Fontein
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import typing as t

from ansible_collections.community.dns.plugins.module_utils._argspec import ArgumentSpec

if t.TYPE_CHECKING:
    from ._provider import ProviderInformation  # pragma: no cover


def create_bulk_operations_argspec(
    provider_information: ProviderInformation,
) -> ArgumentSpec:
    """
    If the provider supports bulk operations, return an ArgumentSpec object with appropriate
    options. Otherwise return an empty one.
    """
    if not provider_information.supports_bulk_actions():
        return ArgumentSpec()

    return ArgumentSpec(
        argument_spec={
            "bulk_operation_threshold": {"type": "int", "default": 2},
        },
    )


def create_record_transformation_argspec() -> ArgumentSpec:
    return ArgumentSpec(
        argument_spec={
            "txt_transformation": {
                "type": "str",
                "default": "unquoted",
                "choices": ["api", "quoted", "unquoted"],
            },
            "txt_character_encoding": {
                "type": "str",
                "default": "decimal",
                "choices": ["decimal", "octal"],
            },
        },
    )



# Copyright (c) 2020, Amin Vakil <info@aminvakil.com>
# Copyright (c) 2016-2018, Matt Davis <mdavis@ansible.com>
# Copyright (c) 2018, Sam Doran <sdoran@redhat.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

import typing as t

from ansible.errors import AnsibleConnectionFailure, AnsibleError
from ansible.module_utils.common.collections import is_string
from ansible.module_utils.common.text.converters import to_native, to_text
from ansible.plugins.action import ActionBase
from ansible.utils.display import Display

if t.TYPE_CHECKING:

    class Distribution(t.TypedDict):
        name: str
        version: str
        family: str


display = Display()


class TimedOutException(Exception):
    pass


class ActionModule(ActionBase):
    TRANSFERS_FILES = False
    _VALID_ARGS = frozenset(("msg", "delay", "search_paths"))

    DEFAULT_CONNECT_TIMEOUT = None
    DEFAULT_PRE_SHUTDOWN_DELAY = 0
    DEFAULT_SHUTDOWN_MESSAGE = "Shut down initiated by Ansible"
    DEFAULT_SHUTDOWN_COMMAND = "shutdown"
    DEFAULT_SHUTDOWN_COMMAND_ARGS = '-h {delay_min} "{message}"'
    DEFAULT_SUDOABLE = True

    SHUTDOWN_COMMANDS = {
        "alpine": "poweroff",
        "vmkernel": "halt",
    }

    SHUTDOWN_COMMAND_ARGS = {
        "alpine": "",
        "void": '-h +{delay_min} "{message}"',
        "freebsd": '-p +{delay_sec}s "{message}"',
        "linux": DEFAULT_SHUTDOWN_COMMAND_ARGS,
        "macosx": '-h +{delay_min} "{message}"',
        "openbsd": '-h +{delay_min} "{message}"',
        "solaris": '-y -g {delay_sec} -i 5 "{message}"',
        "sunos": '-y -g {delay_sec} -i 5 "{message}"',
        "vmkernel": "-d {delay_sec}",
        "aix": "-Fh",
    }

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    @property
    def delay(self):
        return self._check_delay("delay", self.DEFAULT_PRE_SHUTDOWN_DELAY)

    def _check_delay(self, key: str, default: int) -> int:
        """Ensure that the value is positive or zero"""
        value = int(self._task.args.get(key, default))
        if value < 0:
            value = 0
        return value

    @staticmethod
    def _get_value_from_facts(data: dict[str, str], distribution: Distribution, default_value: str) -> str:
        """Get dist+version specific args first, then distribution, then family, lastly use default"""
        return data.get(
            distribution["name"] + distribution["version"],
            data.get(distribution["name"], data.get(distribution["family"], default_value)),
        )

    def get_distribution(self, task_vars: dict[str, t.Any]) -> Distribution:
        # FIXME: only execute the module if we don't already have the facts we need
        display.debug(f"{self._task.action}: running setup module to get distribution")
        module_output = self._execute_module(
            task_vars=task_vars, module_name="ansible.legacy.setup", module_args={"gather_subset": "min"}
        )
        try:
            if module_output.get("failed", False):
                raise AnsibleError(
                    f"Failed to determine system distribution. {to_native(module_output['module_stdout'])}, {to_native(module_output['module_stderr'])}"
                )
            # For some reason, ansible-core insists of using "object" for "arbitrary JSON", which makes it very awkward to use the result...
            ansible_facts: dict[str, t.Any] = module_output["ansible_facts"]  # type: ignore
            distribution: Distribution = {
                "name": ansible_facts["ansible_distribution"].lower(),
                "version": to_text(ansible_facts["ansible_distribution_version"].split(".")[0]),
                "family": to_text(ansible_facts["ansible_os_family"].lower()),
            }
            display.debug(f"{self._task.action}: distribution: {distribution}")
            return distribution
        except KeyError as ke:
            raise AnsibleError(f'Failed to get distribution information. Missing "{ke.args[0]}" in output.') from ke

    def get_shutdown_command(self, task_vars: dict[str, t.Any], distribution: Distribution) -> str:
        def find_command(command: str, find_search_paths: list[str]) -> list[str]:
            display.debug(
                f'{self._task.action}: running find module looking in {find_search_paths} to get path for "{command}"'
            )
            find_result = self._execute_module(
                task_vars=task_vars,
                # prevent collection search by calling with ansible.legacy (still allows library/ override of find)
                module_name="ansible.legacy.find",
                module_args={"paths": find_search_paths, "patterns": [command], "file_type": "any"},
            )
            files: list[dict[str, t.Any]] = find_result["files"]  # type: ignore
            return [x["path"] for x in files]

        shutdown_bin = self._get_value_from_facts(self.SHUTDOWN_COMMANDS, distribution, self.DEFAULT_SHUTDOWN_COMMAND)
        default_search_paths = ["/sbin", "/usr/sbin", "/usr/local/sbin"]
        search_paths = self._task.args.get("search_paths", default_search_paths)

        # FIXME: switch all this to user arg spec validation methods when they are available
        # Convert bare strings to a list
        if is_string(search_paths):
            search_paths = [search_paths]

        try:
            incorrect_type = any(not is_string(x) for x in search_paths)
            if not isinstance(search_paths, list) or incorrect_type:
                raise TypeError
        except TypeError as e:
            # Error if we didn't get a list
            err_msg = f"'search_paths' must be a string or flat list of strings, got {search_paths}"
            raise AnsibleError(err_msg) from e

        full_path = find_command(shutdown_bin, search_paths)  # find the path to the shutdown command
        if not full_path:  # if we could not find the shutdown command
            # tell the user we will try with systemd
            display.vvv(
                f'Unable to find command "{shutdown_bin}" in search paths: {search_paths}, will attempt a shutdown using systemd directly.'
            )
            systemctl_search_paths = ["/bin", "/usr/bin"]
            full_path = find_command("systemctl", systemctl_search_paths)  # find the path to the systemctl command
            if not full_path:  # if we couldn't find systemctl
                raise AnsibleError(
                    f'Could not find command "{shutdown_bin}" in search paths: {search_paths} or systemctl'
                    f" command in search paths: {systemctl_search_paths}, unable to shutdown."
                )  # we give up here
            else:
                return f"{full_path[0]} poweroff"  # done, since we cannot use args with systemd shutdown

        # systemd case taken care of, here we add args to the command
        args = self._get_value_from_facts(self.SHUTDOWN_COMMAND_ARGS, distribution, self.DEFAULT_SHUTDOWN_COMMAND_ARGS)
        # Convert seconds to minutes. If less that 60, set it to 0.
        delay_sec = self.delay
        shutdown_message = self._task.args.get("msg", self.DEFAULT_SHUTDOWN_MESSAGE)

        af = args.format(delay_sec=delay_sec, delay_min=delay_sec // 60, message=shutdown_message)
        return f"{full_path[0]} {af}"

    def perform_shutdown(self, task_vars, distribution) -> dict[str, t.Any]:
        result: dict[str, t.Any] = {}
        shutdown_result: dict[str, t.Any] = {}
        shutdown_command_exec = self.get_shutdown_command(task_vars, distribution)

        self.cleanup(force=True)
        try:
            display.vvv(f"{self._task.action}: shutting down server...")
            display.debug(f"{self._task.action}: shutting down server with command '{shutdown_command_exec}'")
            if self._play_context.check_mode:
                shutdown_result["rc"] = 0
            else:
                shutdown_result.update(
                    self._low_level_execute_command(shutdown_command_exec, sudoable=self.DEFAULT_SUDOABLE)
                )
        except AnsibleConnectionFailure as e:
            # If the connection is closed too quickly due to the system being shutdown, carry on
            display.debug(f"{self._task.action}: AnsibleConnectionFailure caught and handled: {e}")
            shutdown_result["rc"] = 0

        if shutdown_result["rc"] != 0:
            result["failed"] = True
            result["shutdown"] = False
            result["msg"] = (
                f"Shutdown command failed. Error was {to_native(shutdown_result['stdout'])}, {to_native(shutdown_result['stderr'])}"
            )
            return result

        result["failed"] = False
        result["shutdown_command"] = shutdown_command_exec
        return result

    def run(self, tmp: str | None = None, task_vars: dict[str, t.Any] | None = None) -> dict[str, t.Any]:
        self._supports_check_mode = True
        self._supports_async = True

        # If running with local connection, fail so we don't shutdown ourself
        if self._connection.transport == "local" and (not self._play_context.check_mode):
            msg = f"Running {self._task.action} with local connection would shutdown the control node."
            return {"changed": False, "elapsed": 0, "shutdown": False, "failed": True, "msg": msg}

        if task_vars is None:
            task_vars = {}

        result = super().run(tmp, task_vars)

        if result.get("skipped", False) or result.get("failed", False):
            return result

        distribution = self.get_distribution(task_vars)

        # Initiate shutdown
        shutdown_result = self.perform_shutdown(task_vars, distribution)

        if shutdown_result["failed"]:
            result = shutdown_result
            return result

        result["shutdown"] = True
        result["changed"] = True
        result["shutdown_command"] = shutdown_result["shutdown_command"]

        return result



# Copyright (c) 2026, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
name: acme_dns_persist_record
short_description: Craft a DNS record for ACME C(dns-persist-01) challenges
author: Felix Fontein (@felixfontein)
version_added: 3.2.0
description:
  - Craft the content for a ACME C(dns-persist-01) DNS TXT record V(_validation-persist.<domain>).
  - This filter conforms to the L(acme-dns-persist draft 01, https://www.ietf.org/archive/id/draft-ietf-acme-dns-persist-01.html).
    Note that the supported draft version can change at any time,
    and changes will only be considered breaking once the draft reached RFC status.
options:
  _input:
    description:
      - The issuer domain name.
    type: string
    required: true
  account_uri:
    description:
      - The ACME account URI.
    type: string
    required: true
  policy:
    description:
      - The validation scope.
    type: string
    choices:
      wildcard:
        - If this value is present, the CA MAY consider this validation sufficient for issuing certificates
          for the validated FQDN, for specific subdomains of the validated FQDN
          (as covered by wildcard scope or specific subdomain validation rules),
          and for wildcard certificates (for example V(*.example.com)). See
          L(Section 5, https://www.ietf.org/archive/id/draft-ietf-acme-dns-persist-01.html#wildcard-certificate-validation)
          and L(Section 6, https://www.ietf.org/archive/id/draft-ietf-acme-dns-persist-01.html#subdomain-certificate-validation)
          of the L(acme-dns-persist draft 01, https://www.ietf.org/archive/id/draft-ietf-acme-dns-persist-01.html).
  persist_until:
    description:
      - Until when the record is valid.
      - Can be specified as a UNIX time stamp (integer), as a Python datetime object,
        or as a relative time or absolute timestamp specified as a string.
      - Times specified as strings will always be interpreted as UTC.
        Valid format is C([+-]timespec | ASN.1 TIME) where timespec can be an integer
        + C([w | d | h | m | s]) (for example V(+32w1d2h)).
    type: any
seealso:
  - module: community.crypto.acme_certificate
  - module: community.crypto.acme_certificate_order_create
  - module: community.crypto.acme_certificate_order_validate
"""

EXAMPLES = r"""
---
- name: Create _validation-persist.<domain> TXT record contents
  ansible.builtin.debug:
    msg: >-
      {{
        'letsencrypt.org' | community.crypto.acme_dns_persist_record(
          account_uri='https://acme-v02.api.letsencrypt.org/acme/acct/1234',
          policy='wildcard',
          persist_until='+1w',
        )
      }}

- name: Create _validation-persist.<domain> TXT record for example.com
  community.dns.hetzner_dns_record_set:
    prefix: _validation-persist
    zone_name: example.com
    value:
      - >-
        {{
          'letsencrypt.org' | community.crypto.acme_dns_persist_record(
            account_uri='https://acme-v02.api.letsencrypt.org/acme/acct/4321',
            persist_until='20190331202428Z',
          )
        }}
"""

RETURN = r"""
_value:
  description:
    - The content for the V(_validation-persist.<domain>) TXT record.
  type: string
"""

import datetime
import typing as t
from collections.abc import Callable

from ansible.errors import AnsibleFilterError

from ansible_collections.community.crypto.plugins.module_utils._caa import (
    join_issue_value,
)
from ansible_collections.community.crypto.plugins.module_utils._crypto.basic import (
    OpenSSLObjectError,
)
from ansible_collections.community.crypto.plugins.module_utils._time import (
    get_epoch_seconds,
    get_now_datetime,
    get_relative_time_option,
)


def acme_dns_persist_record(
    domain_issuer_name: t.Any,
    *,
    account_uri: t.Any,
    policy: t.Any | None = None,
    persist_until: t.Any | None = None,
) -> str:
    if not isinstance(domain_issuer_name, str):
        raise AnsibleFilterError(
            "The input for the community.crypto.acme_dns_persist_record filter"
            f" must be a string; got {type(domain_issuer_name)} instead"
        )
    if not isinstance(account_uri, str):
        raise AnsibleFilterError(
            "The account_uri parameter for the community.crypto.acme_dns_persist_record filter"
            f" must be a string; got {type(account_uri)} instead"
        )
    valid_policies = ("wildcard",)
    if policy is not None and policy not in valid_policies:
        choices = ", ".join(f'"{vp}"' for vp in valid_policies)
        raise AnsibleFilterError(
            "The policy parameter for the community.crypto.acme_dns_persist_record filter"
            f" must be one of {choices}; got {policy!r} instead"
        )
    if persist_until is not None:
        if isinstance(persist_until, str):
            try:
                persist_until = get_relative_time_option(
                    persist_until,
                    input_name="persist_until",
                    with_timezone=True,
                    now=get_now_datetime(with_timezone=True),
                )
            except OpenSSLObjectError as exc:
                raise AnsibleFilterError(
                    "Error parsing persist_until parameter for the community.crypto.acme_dns_persist_record filter:"
                    f" {exc}"
                ) from None
        if isinstance(persist_until, int) and not isinstance(persist_until, bool):
            pass
        elif isinstance(persist_until, datetime.datetime):
            persist_until = int(get_epoch_seconds(persist_until))
        else:
            raise AnsibleFilterError(
                "The persist_until parameter for the community.crypto.acme_dns_persist_record filter"
                f" must be an integer, a string, or a datetime object; got {type(persist_until)} instead"
            )

    parts = [("accounturi", account_uri)]
    if policy is not None:
        parts.append(("policy", policy))
    if persist_until is not None:
        parts.append(("persistUntil", str(persist_until)))
    try:
        return join_issue_value(domain_issuer_name, parts)
    except ValueError as exc:
        raise AnsibleFilterError(
            "Error composing result for the community.crypto.acme_dns_persist_record filter:"
            f" {exc}"
        ) from exc


class FilterModule:
    """Ansible jinja2 filters"""

    def filters(self) -> dict[str, Callable]:
        return {
            "acme_dns_persist_record": acme_dns_persist_record,
        }



#!/usr/bin/python

# Copyright (c) 2021, Werner Dijkerman (ikben@werner-dijkerman.nl)
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later
from __future__ import annotations

DOCUMENTATION = r"""
module: gitlab_branch
short_description: Create or delete a branch
version_added: 4.2.0
description:
  - This module allows to create or delete branches.
author:
  - paytroff (@paytroff)
requirements:
  - python-gitlab >= 2.3.0
extends_documentation_fragment:
  - community.general._auth_basic
  - community.general._gitlab
  - community.general._attributes

attributes:
  check_mode:
    support: none
  diff_mode:
    support: none

options:
  state:
    description:
      - Create or delete branch.
    default: present
    type: str
    choices: ["present", "absent"]
  project:
    description:
      - The path or name of the project.
    required: true
    type: str
  branch:
    description:
      - The name of the branch that needs to be created.
    required: true
    type: str
  ref_branch:
    description:
      - Reference branch to create from.
      - This must be specified if O(state=present).
    type: str
"""


EXAMPLES = r"""
- name: Create branch branch2 from main
  community.general.gitlab_branch:
    api_url: https://gitlab.com
    api_token: secret_access_token
    project: "group1/project1"
    branch: branch2
    ref_branch: main
    state: present

- name: Delete branch branch2
  community.general.gitlab_branch:
    api_url: https://gitlab.com
    api_token: secret_access_token
    project: "group1/project1"
    branch: branch2
    state: absent
"""

RETURN = r"""
"""

import traceback

from ansible.module_utils.api import basic_auth_argument_spec
from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._gitlab import (
    auth_argument_spec,
    gitlab,
    gitlab_authentication,
)
from ansible_collections.community.general.plugins.module_utils._version import LooseVersion


class GitlabBranch:
    def __init__(self, module, project, gitlab_instance):
        self.repo = gitlab_instance
        self._module = module
        self.project = self.get_project(project)

    def get_project(self, project):
        try:
            return self.repo.projects.get(project)
        except Exception:
            return False

    def get_branch(self, branch):
        try:
            return self.project.branches.get(branch)
        except Exception:
            return False

    def create_branch(self, branch, ref_branch):
        return self.project.branches.create({"branch": branch, "ref": ref_branch})

    def delete_branch(self, branch):
        return branch.delete()


def main():
    argument_spec = basic_auth_argument_spec()
    argument_spec.update(auth_argument_spec())
    argument_spec.update(
        project=dict(type="str", required=True),
        branch=dict(type="str", required=True),
        ref_branch=dict(type="str"),
        state=dict(type="str", default="present", choices=["absent", "present"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        mutually_exclusive=[
            ["api_username", "api_token"],
            ["api_username", "api_oauth_token"],
            ["api_username", "api_job_token"],
            ["api_token", "api_oauth_token"],
            ["api_token", "api_job_token"],
        ],
        required_together=[
            ["api_username", "api_password"],
        ],
        required_one_of=[["api_username", "api_token", "api_oauth_token", "api_job_token"]],
        required_if=[
            ["state", "present", ["ref_branch"], True],
        ],
        supports_check_mode=False,
    )

    # check prerequisites and connect to gitlab server
    gitlab_instance = gitlab_authentication(module)

    project = module.params["project"]
    branch = module.params["branch"]
    ref_branch = module.params["ref_branch"]
    state = module.params["state"]

    gitlab_version = gitlab.__version__
    if LooseVersion(gitlab_version) < LooseVersion("2.3.0"):
        module.fail_json(
            msg=f"community.general.gitlab_branch requires python-gitlab Python module >= 2.3.0 (installed version: [{gitlab_version}])."
            " Please upgrade python-gitlab to version 2.3.0 or above."
        )

    this_gitlab = GitlabBranch(module=module, project=project, gitlab_instance=gitlab_instance)

    this_branch = this_gitlab.get_branch(branch)

    if not this_branch and state == "present":
        r_branch = this_gitlab.get_branch(ref_branch)
        if not r_branch:
            module.fail_json(msg=f"Ref branch {ref_branch} not exist.")
        this_gitlab.create_branch(branch, ref_branch)
        module.exit_json(changed=True, msg=f"Created the branch {branch}.")
    elif this_branch and state == "present":
        module.exit_json(changed=False, msg=f"Branch {branch} already exist")
    elif this_branch and state == "absent":
        try:
            this_gitlab.delete_branch(this_branch)
            module.exit_json(changed=True, msg=f"Branch {branch} deleted.")
        except Exception:
            module.fail_json(msg="Error delete branch.", exception=traceback.format_exc())
    else:
        module.exit_json(changed=False, msg="No changes are needed.")


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2017 Zim Kalinowski, <zikalino@microsoft.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_containerregistryreplication
version_added: "0.1.2"
short_description: Manage Replication instance.
description:
    - Create, update and delete instance of Replication.

options:
    resource_group:
        description:
            - The name of the resource group to which the container registry belongs.
        required: True
        type: str
    registry_name:
        description:
            - The name of the container registry.
        required: True
        type: str
    replication_name:
        description:
            - The name of the I(replication).
        required: True
        type: str
    replication:
        description:
            - The parameters for creating a replication.
        type: dict
    location:
        description:
            - Resource location. If not set, location from the resource group will be used as default.
        type: str
    state:
        description:
            - Assert the state of the Container registery replication.
             - Use C(present) to create or update Container registery replication and C(absent) to delete it.
        default: present
        type: str
        choices:
            - absent
            - present

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - "Zim Kalinowski (@zikalino)"

'''

EXAMPLES = '''
- name: Create (or update) Replication
  azure_rm_containerregistryreplication:
    resource_group: myResourceGroup
    registry_name: myRegistry
    replication_name: myReplication
    replication:
      Key: value
    location: eastus
'''

RETURN = '''
id:
    description:
        - The resource ID.
    returned: always
    type: str
    sample: "/subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/myResourceGroup/providers/Microsoft.ContainerRegistry/registries/myRegistry/r
            eplications/myReplication"
status:
    description:
        - The status of the replication at the time the operation was called.
    returned: always
    type: str
    sample: Ready
'''

import time
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.core.exceptions import ResourceNotFoundError
    from azure.core.polling import LROPoller
    from azure.mgmt.containerregistry.models import Replication, ReplicationUpdateParameters
except ImportError:
    # This is handled in azure_rm_common
    pass


def create_replication_dict(replication):
    if replication is None:
        return None
    results = dict(
        id=replication.id,
        name=replication.name,
        location=replication.location,
        provisioning_state=replication.provisioning_state,
        tags=replication.tags,
        status=replication.status.display_status
    )
    return results


class Actions:
    NoAction, Create, Update, Delete = range(4)


class AzureRMReplications(AzureRMModuleBase):
    """Configuration class for an Azure RM Replication resource"""

    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            registry_name=dict(
                type='str',
                required=True
            ),
            replication_name=dict(
                type='str',
                required=True
            ),
            replication=dict(
                type='dict'
            ),
            location=dict(
                type='str'
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent']
            )
        )

        self.resource_group = None
        self.registry_name = None
        self.replication_name = None
        self.location = None

        self.results = dict(changed=False)
        self.state = None
        self.to_do = Actions.NoAction

        super(AzureRMReplications, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                  supports_check_mode=True,
                                                  supports_tags=False)

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for key in list(self.module_arg_spec.keys()):
            if hasattr(self, key):
                setattr(self, key, kwargs[key])

        old_response = None
        response = None

        resource_group = self.get_resource_group(self.resource_group)

        if self.location is None:
            self.location = resource_group.location

        old_response = self.get_replication()

        if not old_response:
            self.log("Replication instance doesn't exist")
            if self.state == 'absent':
                self.log("Old instance didn't exist")
            else:
                self.to_do = Actions.Create
        else:
            self.log("Replication instance already exists")
            if self.state == 'absent':
                self.to_do = Actions.Delete
            elif self.state == 'present':
                self.log("Need to check if Replication instance has to be deleted or may be updated")
                self.to_do = Actions.Update

        if (self.to_do == Actions.Create) or (self.to_do == Actions.Update):
            self.log("Need to Create / Update the Replication instance")

            if self.check_mode:
                self.results['changed'] = True
                return self.results

            response = self.create_update_replication()

            if not old_response:
                self.results['changed'] = True
            else:
                self.results['changed'] = old_response.__ne__(response)
            self.log("Creation / Update done")
        elif self.to_do == Actions.Delete:
            self.log("Replication instance deleted")
            self.results['changed'] = True

            if self.check_mode:
                return self.results

            self.delete_replication()
            # make sure instance is actually deleted, for some Azure resources, instance is hanging around
            # for some time after deletion -- this should be really fixed in Azure
            while self.get_replication():
                time.sleep(20)
        else:
            self.log("Replication instance unchanged")
            self.results['changed'] = False
            response = old_response

        if response:
            self.results["id"] = response["id"]
            self.results["status"] = response["status"]

        return self.results

    def create_update_replication(self):
        '''
        Creates or updates Replication with the specified configuration.

        :return: deserialized Replication instance state dictionary
        '''
        self.log("Creating / Updating the Replication instance {0}".format(self.replication_name))

        try:
            if self.to_do == Actions.Create:
                replication = Replication(
                    location=self.location,
                )
                response = self.containerregistry_client.replications.begin_create(resource_group_name=self.resource_group,
                                                                                   registry_name=self.registry_name,
                                                                                   replication_name=self.replication_name,
                                                                                   replication=replication)
            else:
                update_params = ReplicationUpdateParameters()
                response = self.containerregistry_client.replications.begin_update(resource_group_name=self.resource_group,
                                                                                   registry_name=self.registry_name,
                                                                                   replication_name=self.replication_name,
                                                                                   replication_update_parameters=update_params)
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)

        except Exception as exc:
            self.log('Error attempting to create the Replication instance.')
            self.fail("Error creating the Replication instance: {0}".format(str(exc)))
        return create_replication_dict(response)

    def delete_replication(self):
        '''
        Deletes specified Replication instance in the specified subscription and resource group.

        :return: True
        '''
        self.log("Deleting the Replication instance {0}".format(self.replication_name))
        try:
            response = self.containerregistry_client.replications.begin_delete(resource_group_name=self.resource_group,
                                                                               registry_name=self.registry_name,
                                                                               replication_name=self.replication_name)
            self.get_poller_result(response)
        except Exception as e:
            self.log('Error attempting to delete the Replication instance.')
            self.fail("Error deleting the Replication instance: {0}".format(str(e)))

        return True

    def get_replication(self):
        '''
        Gets the properties of the specified Replication.

        :return: deserialized Replication instance state dictionary
        '''
        self.log("Checking if the Replication instance {0} is present".format(self.replication_name))
        found = False
        try:
            response = self.containerregistry_client.replications.get(resource_group_name=self.resource_group,
                                                                      registry_name=self.registry_name,
                                                                      replication_name=self.replication_name)
            found = True
            self.log("Response : {0}".format(response))
            self.log("Replication instance : {0} found".format(response.name))
        except ResourceNotFoundError as e:
            self.log('Did not find the Replication instance: {0}'.format(str(e)))
        if found is True:
            return response.as_dict()

        return False


def main():
    """Main execution"""
    AzureRMReplications()


if __name__ == '__main__':
    main()



#!/usr/bin/python

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

DOCUMENTATION = """
---
module: facts
author: "Petr Klima (@qaxi)"
short_description: Collect facts from remote devices running Cisco SMB
description:
  - Collects a base set of device facts from a remote device that
    is running Cisco SMB.  This module prepends all of the
    base network fact keys with C(ansible_net_<fact>).  The facts
    module will always collect a base set of facts from the device
    and can enable or disable collection of additional facts.
options:
  gather_subset:
    description:
      - When supplied, this argument will restrict the facts collected
        to a given subset.  Possible values for this argument include
        C(all), C(hardware), C(config) and C(interfaces).  Can specify a list of
        values to include a larger subset.  Values can also be used
        with an initial C(!) to specify that a specific subset should
        not be collected.
    required: false
    type: list
    elements: str
    choices: [ 'default', 'all', 'hardware', 'config', 'interfaces', '!hardware', '!config', '!interfaces' ]
    default: '!config'
notes:
  - Supports C(check_mode).
"""

EXAMPLES = """
- name: Collect all facts from the device
  community.ciscosmb.facts:
    gather_subset: all

- name: Collect only the config and default facts
  community.ciscosmb.facts:
    gather_subset:
      - config

- name: Do not collect hardware facts
  community.ciscosmb.facts:
    gather_subset:
      - "!hardware"
"""

RETURN = """
ansible_net_gather_subset:
  description: The list of fact subsets collected from the device.
  returned: always
  type: list

# default
ansible_net_model:
  description: The model name returned from the device.
  returned: always
  type: str
ansible_net_serialnum:
  description: The serial number of the remote device.
  returned: always
  type: str
ansible_net_version:
  description: The operating system version running on the remote device.
  returned: always
  type: str
ansible_net_hostname:
  description: The configured hostname of the device.
  returned: always
  type: str
ansible_net_uptime:
  description: The uptime of the device.
  returned: always
  type: str
ansible_net_cpu_load:
  description: Current CPU load.
  returned: always
  type: str
ansible_net_stacked_models:
  description: The model names of each device in the stack.
  returned: when multiple devices are configured in a stack
  type: list
ansible_net_stacked_serialnums:
  description: The serial numbers of each device in the stack.
  returned: when multiple devices are configured in a stack
  type: list

# hardware
ansible_net_spacefree_mb:
  description: The available disk space on the remote device in MiB.
  returned: when hardware is configured
  type: dict
ansible_net_spacetotal_mb:
  description: The total disk space on the remote device in MiB.
  returned: when hardware is configured
  type: dict
ansible_net_memfree_mb:
  description: The available free memory on the remote device in MiB.
  returned: when hardware is configured
  type: int
ansible_net_memtotal_mb:
  description: The total memory on the remote device in MiB.
  returned: when hardware is configured
  type: int

# config
ansible_net_config:
  description: The current active config from the device.
  returned: when config is configured
  type: str

# interfaces
ansible_net_all_ipv4_addresses:
  description: All IPv4 addresses configured on the device.
  returned: when interfaces is configured
  type: list
ansible_net_all_ipv6_addresses:
  description: All IPv6 addresses configured on the device.
  returned: when interfaces is configured
  type: list
ansible_net_interfaces:
  description: A hash of all interfaces running on the system.
  returned: when interfaces is configured
  type: dict
ansible_net_neighbors:
  description: The list of neighbors from the remote device.
  returned: when interfaces is configured
  type: dict

"""
import re

from ansible_collections.community.ciscosmb.plugins.module_utils.ciscosmb import (
    run_commands,
    ciscosmb_argument_spec,
    interface_canonical_name,
    ciscosmb_split_to_tables,
    ciscosmb_parse_table,
    ciscosmb_merge_dicts,
)
from ansible.module_utils.basic import AnsibleModule


class FactsBase(object):
    COMMANDS = list()

    def __init__(self, module):
        self.module = module
        self.facts = dict()
        self.responses = None

    def populate(self):
        self.responses = run_commands(
            self.module, commands=self.COMMANDS, check_rc=False
        )

    def run(self, cmd):
        return run_commands(self.module, commands=cmd, check_rc=False)


class Default(FactsBase):
    COMMANDS = [
        "show version",
        "show system",
        "show cpu utilization",
        "show inventory",
    ]

    def populate(self):
        super(Default, self).populate()

        data = self.responses[0]
        if data:
            self.facts["version"] = self.parse_version(data)
            self.facts["boot_version"] = self.parse_boot_version(data)

        data = self.responses[1]
        if data:
            self.facts["uptime"] = self.parse_uptime(data)
            self.facts["hostname"] = self.parse_hostname(data)

        data = self.responses[2]
        if data:
            self.facts["cpu_load"] = self.parse_cpu_load(data)

        data = self.responses[3]
        if data:
            modules = self.parse_inventory(data)
            stacked_models = self.parse_stacked_models(modules)
            if len(stacked_models) >= 2:
                stacked_serialnums = self.parse_stacked_serialnums(modules)
                self.facts["stacked_models"] = stacked_models
                self.facts["stacked_serialnums"] = stacked_serialnums
            self.facts["model"] = self.parse_model(modules)
            self.facts["serialnum"] = self.parse_serialnum(modules)
            self.facts["hw_version"] = self.parse_hw_version(modules)
            self.facts["hw_modules"] = modules

    # show version
    def parse_version(self, data):
        # Cisco SMB 300 and 500 - fw 1.x.x.x
        match = re.search(r"^SW version\s*(\S+)\s*.*$", data, re.M)
        if match:
            return match.group(1)
        # Cisco SMB 350 and 550 - fw 2.x.x.x
        match = re.search(r"^  Version:\s*(\S+)\s*.*$", data, re.M)
        if match:
            return match.group(1)

    def parse_boot_version(self, data):
        match = re.search(r"Boot version\s*(\S+)\s*.*$", data, re.M)
        if match:
            return match.group(1)

    # show system
    def parse_uptime(self, data):
        match = re.search(r"^System Up Time \S+:\s+(\S+)\s*$", data, re.M)
        if match:
            (dayhour, mins, sec) = match.group(1).split(":")
            (day, hour) = dayhour.split(",")
            # output in seconds
            return (int(day) * 86400) + (int(hour) * 3600) + (int(mins) * 60) + int(sec)

    def parse_hostname(self, data):
        match = re.search(r"^System Name:\s*(\S+)\s*$", data, re.M)
        if match:
            return match.group(1)

    # show cpu utilization
    def parse_cpu_load(self, data):
        match = re.search(r"one minute:\s+(\d+)%;\s*", data, re.M)
        if match:
            return match.group(1)

    # show inventory
    def parse_inventory(self, data):
        # make 1 module 1 line
        data = re.sub(r"\nPID", "  PID", data)
        # delete empty lines
        data = re.sub(r"^\n", "", data)
        data = re.sub(r"\n\n", "", data)
        data = re.sub(r"\n\s*\n", r"\n", data)

        lines = data.splitlines()

        modules = {}
        for line in lines:
            # remove extra chars
            line = re.sub(r'"', r"", line)
            line = re.sub(r"\s+", r" ", line)
            # normalize lines
            line = re.sub(r":\s", r'"', line)
            line = re.sub(r'\s+DESCR"', r'"DESCR"', line)
            line = re.sub(r'\s+PID"', r'"PID"', line)
            line = re.sub(r'\s+VID"', r'"VID"', line)
            line = re.sub(r'\s+SN"', r'"SN"', line)
            line = re.sub(r"\s*$", r"", line)

            match = re.search(
                r'^NAME"(?P<name>[^"]+)"DESCR"(?P<descr>[^"]+)"PID"(?P<pid>[^"]+)"VID"(?P<vid>[^"]+)"SN"(?P<sn>\S+)\s*',
                line,
            )

            if match:
                modul = match.groupdict()
                modules[modul["name"]] = modul

        if modules:
            return modules

    def parse_stacked_models(self, data):
        # every inventory has module with NAME: "1"
        # stacks have modules 2 3 ... 8
        models = []
        for n in range(1, 9):
            # index is string
            if str(n) in data:
                models.append(data[str(n)]["pid"])
        return models

    def parse_stacked_serialnums(self, data):
        # every inventory has module with NAME: "1"
        # stacks have modules 2 3 ... 8
        sn = []
        for n in range(1, 9):
            # index is string
            if str(n) in data:
                sn.append(data[str(n)]["sn"])
        return sn

    def parse_model(self, data):
        # every inventory has module with NAME: "1"
        model = data["1"]["pid"]
        if "stacked_models" in self.facts:
            model = re.sub(r"-.*$", "", model)
            model = "Stack " + model
        return model

    def parse_serialnum(self, data):
        # every inventory has module with NAME: "1"
        sn = data["1"]["sn"]
        return sn

    def parse_hw_version(self, data):
        # every inventory has module with NAME: "1"
        sn = data["1"]["vid"]
        return sn


class Hardware(FactsBase):
    COMMANDS = [
        "dir",
    ]

    def populate(self):
        super(Hardware, self).populate()
        data = self.responses[0]
        if data:
            self.parse_filesystem_info(data)

    def parse_filesystem_info(self, data):
        match = re.search(r"Total size of (\S+): (\d+) bytes", data, re.M)

        if match:  # fw 1.x
            self.facts["spacetotal_mb"] = round(int(match.group(2)) / 1024 / 1024, 1)
            match = re.search(r"Free size of (\S+): (\d+) bytes", data, re.M)
            self.facts["spacefree_mb"] = round(int(match.group(2)) / 1024 / 1024, 1)

        else:
            match = re.search(r"(\d+)K of (\d+)K are free", data, re.M)
            if match:  # fw 2.x, 3.x
                self.facts["spacetotal_mb"] = round(int(match.group(2)) / 1024, 1)
                self.facts["spacefree_mb"] = round(int(match.group(1)) / 1024, 1)


class Config(FactsBase):
    COMMANDS = ["show running-config detailed"]

    def populate(self):
        super(Config, self).populate()
        data = self.responses[0]
        if data:
            self.facts["config"] = data


class Interfaces(FactsBase):
    COMMANDS = [
        "show ports jumbo-frame",
        "show ip interface",
        "show ipv6 interface brief",
        "show interfaces status",
        "show interfaces configuration",
        "show interfaces description",
        "show lldp neighbors",
    ]

    DETAIL_RE = re.compile(
        r"([\w\d\-]+)=\"?(\w{3}/\d{2}/\d{4}\s\d{2}:\d{2}:\d{2}|[\w\d\-\.:/]+)"
    )
    WRAPPED_LINE_RE = re.compile(r"^\s+(?!\d)")

    def populate(self):
        super(Interfaces, self).populate()

        self.facts["interfaces"] = dict()
        self.facts["all_ipv4_addresses"] = list()
        self.facts["all_ipv6_addresses"] = list()
        self.facts["neighbors"] = list()

        data = self.responses[0]
        if data:
            self.populate_interfaces_mtu(data)

        data = self.responses[1]
        if data:
            self.populate_addresses_ipv4(data)

        data = self.responses[2]
        if data:
            self.populate_addresses_ipv6(data)

        data = self.responses[3]
        if data:
            self.populate_interfaces_status(data)

        data = self.responses[4]
        if data:
            self.populate_interfaces_configuration(data)

        data = self.responses[5]
        if data:
            self.populate_interfaces_description(data)

        data = self.responses[6]
        if data:
            self.populate_neighbors(data)

    def _populate_interfaces_status_interface(self, interface_table):
        interfaces = dict()

        for key in interface_table:
            i = interface_table[key]
            interface = dict()
            interface["state"] = i[6].lower()
            interface["type"] = i[1]
            interface["mtu"] = self._mtu
            interface["duplex"] = i[2].lower()
            interface["negotiation"] = i[4].lower()
            interface["control"] = i[5].lower()
            interface["presure"] = i[7].lower()
            interface["mode"] = i[8].lower()

            if i[6] == "Up":
                interface["bandwidth"] = int(i[3]) * 1000  # to get speed in kb
                interface["bandwith"] = interface["bandwidth"]
            else:
                interface["bandwidth"] = None
                interface["bandwith"] = interface["bandwidth"]

            for key in interface:
                if interface[key] == "--":
                    interface[key] = None

            interfaces[interface_canonical_name(i[0])] = interface
        return interfaces

    def _populate_interfaces_status_portchanel(self, interface_table):
        interfaces = dict()

        for key in interface_table:
            interface = dict()
            i = interface_table[key]
            interface["state"] = i[6].lower()
            interface["type"] = i[1]
            interface["mtu"] = self._mtu
            interface["duplex"] = i[2].lower()
            interface["negotiation"] = i[4].lower()
            interface["control"] = i[5].lower()

            if i[6] == "Up":
                interface["bandwidth"] = int(i[3]) * 1000  # to get speed in kb
                interface["bandwith"] = interface["bandwidth"]
            else:
                interface["bandwidth"] = None
                interface["bandwith"] = interface["bandwidth"]

            for key in interface:
                if interface[key] == "--":
                    interface[key] = None

            interfaces[interface_canonical_name(i[0])] = interface

        return interfaces

    def populate_interfaces_status(self, data):
        tables = ciscosmb_split_to_tables(data)

        interface_table = ciscosmb_parse_table(tables[0])
        portchanel_table = ciscosmb_parse_table(tables[1])

        interfaces = self._populate_interfaces_status_interface(interface_table)
        self.facts["interfaces"] = ciscosmb_merge_dicts(
            self.facts["interfaces"], interfaces
        )
        interfaces = self._populate_interfaces_status_portchanel(portchanel_table)
        self.facts["interfaces"] = ciscosmb_merge_dicts(
            self.facts["interfaces"], interfaces
        )

    def _populate_interfaces_configuration_interface(self, interface_table):
        interfaces = dict()

        for key in interface_table:
            i = interface_table[key]
            interface = dict()
            interface["admin_state"] = i[6].lower()
            interface["mdix"] = i[8].lower()

            interfaces[interface_canonical_name(i[0])] = interface
        return interfaces

    def _populate_interfaces_configuration_portchanel(self, interface_table):
        interfaces = dict()

        for key in interface_table:
            interface = dict()
            i = interface_table[key]

            interface["admin_state"] = i[5].lower()

            interfaces[interface_canonical_name(i[0])] = interface

        return interfaces

    def populate_interfaces_configuration(self, data):
        tables = ciscosmb_split_to_tables(data)

        interface_table = ciscosmb_parse_table(tables[0])
        portchanel_table = ciscosmb_parse_table(tables[1])

        interfaces = self._populate_interfaces_configuration_interface(interface_table)
        self.facts["interfaces"] = ciscosmb_merge_dicts(
            self.facts["interfaces"], interfaces
        )
        interfaces = self._populate_interfaces_configuration_portchanel(
            portchanel_table
        )
        self.facts["interfaces"] = ciscosmb_merge_dicts(
            self.facts["interfaces"], interfaces
        )

    def _populate_interfaces_description_interface(self, interface_table):
        interfaces = dict()

        for key in interface_table:
            i = interface_table[key]
            interface = dict()
            interface["description"] = i[1]

            if interface["description"] == "":
                interface["description"] = None

            interfaces[interface_canonical_name(i[0])] = interface
        return interfaces

    def _populate_interfaces_description_portchanel(self, interface_table):
        interfaces = dict()

        for key in interface_table:
            interface = dict()
            i = interface_table[key]

            interface["description"] = i[1]

            if interface["description"] == "":
                interface["description"] = None

            interfaces[interface_canonical_name(i[0])] = interface

        return interfaces

    def populate_interfaces_description(self, data):
        tables = ciscosmb_split_to_tables(data)

        interface_table = ciscosmb_parse_table(tables[0], False)
        portchanel_table = ciscosmb_parse_table(tables[1], False)

        interfaces = self._populate_interfaces_description_interface(interface_table)
        self.facts["interfaces"] = ciscosmb_merge_dicts(
            self.facts["interfaces"], interfaces
        )
        interfaces = self._populate_interfaces_description_portchanel(portchanel_table)
        self.facts["interfaces"] = ciscosmb_merge_dicts(
            self.facts["interfaces"], interfaces
        )

    def _populate_address_ipv4(self, ip_table):
        ips = list()

        for key in ip_table:
            cidr = ip_table[key][0]

            interface = interface_canonical_name(ip_table[key][1])
            ip, mask = cidr.split("/")

            ips.append(ip)

            # add ips to interface
            self._new_interface(interface)
            if "ipv4" not in self.facts["interfaces"][interface]:
                self.facts["interfaces"][interface]["ipv4"] = list()

            self.facts["interfaces"][interface]["ipv4"].append(
                dict(address=ip, subnet=mask)
            )

        return ips

    def populate_addresses_ipv4(self, data):
        tables = ciscosmb_split_to_tables(data)
        ip_table = ciscosmb_parse_table(tables[0])

        ips = self._populate_address_ipv4(ip_table)
        self.facts["all_ipv4_addresses"] = ips

    def _populate_address_ipv6(self, ip_table):
        ips = list()

        for key in ip_table:
            ip = ip_table[key][3]
            interface = interface_canonical_name(ip_table[key][0])

            ips.append(ip)

            # add ips to interface
            self._new_interface(interface)
            if "ipv6" not in self.facts["interfaces"][interface]:
                self.facts["interfaces"][interface]["ipv6"] = list()

            self.facts["interfaces"][interface]["ipv6"].append(dict(address=ip))

        return ips

    def _new_interface(self, interface):
        if interface in self.facts["interfaces"]:
            return
        else:
            self.facts["interfaces"][interface] = dict()
            self.facts["interfaces"][interface]["mtu"] = self._mtu
            self.facts["interfaces"][interface]["admin_state"] = "up"
            self.facts["interfaces"][interface]["description"] = None
            self.facts["interfaces"][interface]["state"] = "up"
            self.facts["interfaces"][interface]["bandwidth"] = None
            self.facts["interfaces"][interface]["bandwith"] = None
            self.facts["interfaces"][interface]["duplex"] = None
            self.facts["interfaces"][interface]["negotiation"] = None
            self.facts["interfaces"][interface]["control"] = None
            return

    def populate_addresses_ipv6(self, data):
        tables = ciscosmb_split_to_tables(data)

        ip_table = ciscosmb_parse_table(tables[0])
        ips = self._populate_address_ipv6(ip_table)
        self.facts["all_ipv6_addresses"] = ips

    def populate_interfaces_mtu(self, data):
        # by documentation SG350
        match = re.search(r"Jumbo frames are enabled", data, re.M)
        if match:
            mtu = 9000
        else:
            mtu = 1518

        self._mtu = mtu

    def populate_neighbors(self, data):
        tables = ciscosmb_split_to_tables(data)

        neighbor_table = ciscosmb_parse_table(tables[0], allow_empty_fields=[3, 4])

        neighbors = dict()
        for key in neighbor_table:
            neighbor = neighbor_table[key]

            ifcname = interface_canonical_name(neighbor[0])

            host = neighbor[3]
            port = neighbor[2]

            hostport = {"host": host, "port": port}

            if ifcname not in neighbors:
                neighbors[ifcname] = list()

            neighbors[ifcname].append(hostport)

        self.facts["neighbors"] = neighbors


FACT_SUBSETS = dict(
    default=Default,
    hardware=Hardware,
    interfaces=Interfaces,
    config=Config,
)

VALID_SUBSETS = frozenset(FACT_SUBSETS.keys())

warnings = list()


def main():
    """main entry point for module execution"""
    argument_spec = dict(
        gather_subset=dict(
            default=["!config"],
            type="list",
            elements="str",
            choices=[
                "all",
                "default",
                "hardware",
                "interfaces",
                "config",
                "!hardware",
                "!interfaces",
                "!config",
            ],
        )
    )

    argument_spec.update(ciscosmb_argument_spec)

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)

    gather_subset = module.params["gather_subset"]

    runable_subsets = set()
    exclude_subsets = set()

    for subset in gather_subset:
        if subset == "all":
            runable_subsets.update(VALID_SUBSETS)
            continue

        if subset.startswith("!"):
            subset = subset[1:]
            if subset == "all":
                exclude_subsets.update(VALID_SUBSETS)
                continue
            exclude = True
        else:
            exclude = False

        if subset not in VALID_SUBSETS:
            module.fail_json(msg="Bad subset: %s" % subset)

        if exclude:
            exclude_subsets.add(subset)
        else:
            runable_subsets.add(subset)

    if not runable_subsets:
        runable_subsets.update(VALID_SUBSETS)

    runable_subsets.difference_update(exclude_subsets)
    runable_subsets.add("default")

    facts = dict()
    facts["gather_subset"] = list(runable_subsets)

    instances = list()
    for key in runable_subsets:
        instances.append(FACT_SUBSETS[key](module))

    for inst in instances:
        inst.populate()
        facts.update(inst.facts)

    ansible_facts = dict()
    for key, value in facts.items():
        key = "ansible_net_%s" % key
        ansible_facts[key] = value

    module.exit_json(ansible_facts=ansible_facts, warnings=warnings)


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2024, Michael Ilg
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: consul_agent_check
short_description: Add, modify, and delete checks within a Consul cluster
version_added: 9.1.0
description:
  - Allows the addition, modification and deletion of checks in a Consul cluster using the agent. For more details on using
    and configuring Checks, see U(https://developer.hashicorp.com/consul/api-docs/agent/check).
  - Currently, there is no complete way to retrieve the script, interval or TTL metadata for a registered check. Without this
    metadata it is not possible to tell if the data supplied with ansible represents a change to a check. As a result, the
    module does not attempt to determine changes and it always reports a changed occurred. An API method is planned to supply
    this metadata so at that stage change management is to be added.
author:
  - Michael Ilg (@Ilgmi)
extends_documentation_fragment:
  - community.general._consul
  - community.general._consul.actiongroup_consul
  - community.general._consul.token
  - community.general._attributes
attributes:
  check_mode:
    support: full
    details:
      - The result is the object as it is defined in the module options and not the object structure of the Consul API. For
        a better overview of what the object structure looks like, take a look at U(https://developer.hashicorp.com/consul/api-docs/agent/check#list-checks).
  diff_mode:
    support: partial
    details:
      - In check mode the diff shows the object as it is defined in the module options and not the object structure of the
        Consul API.
options:
  state:
    description:
      - Whether the check should be present or absent.
    choices: ['present', 'absent']
    default: present
    type: str
  name:
    description:
      - Required name for the service check.
    type: str
  id:
    description:
      - Specifies a unique ID for this check on the node. This defaults to the O(name) parameter, but it may be necessary
        to provide an ID for uniqueness. This value is returned in the response as V(CheckId).
    type: str
  interval:
    description:
      - The interval at which the service check is run. This is a number with a V(s) or V(m) suffix to signify the units of
        seconds or minutes, for example V(15s) or V(1m). If no suffix is supplied V(s) is used by default, for example V(10)
        is equivalent to V(10s).
      - Required if one of the parameters O(args), O(http), or O(tcp) is specified.
    type: str
  notes:
    description:
      - Notes to attach to check when registering it.
    type: str
  args:
    description:
      - Specifies command arguments to run to update the status of the check.
      - Requires O(interval) to be provided.
      - Mutually exclusive with O(ttl), O(tcp) and O(http).
    type: list
    elements: str
  ttl:
    description:
      - Checks can be registered with a TTL instead of a O(args) and O(interval) this means that the service checks in with
        the agent before the TTL expires. If it does not the check is considered failed. Required if registering a check and
        the script an interval are missing Similar to the interval this is a number with a V(s) or V(m) suffix to signify
        the units of seconds or minutes, for example V(15s) or V(1m). If no suffix is supplied V(s) is used by default, for
        example V(10) is equivalent to V(10s).
      - Mutually exclusive with O(args), O(tcp) and O(http).
    type: str
  tcp:
    description:
      - Checks can be registered with a TCP port. This means that Consul will check if the connection attempt to that port
        is successful (that is, the port is currently accepting connections). The format is V(host:port), for example V(localhost:80).
      - Requires O(interval) to be provided.
      - Mutually exclusive with O(args), O(ttl) and O(http).
    type: str
    version_added: '1.3.0'
  http:
    description:
      - Checks can be registered with an HTTP endpoint. This means that Consul checks that the HTTP endpoint returns a successful
        HTTP status.
      - Requires O(interval) to be provided.
      - Mutually exclusive with O(args), O(ttl) and O(tcp).
    type: str
  timeout:
    description:
      - A custom HTTP check timeout. The Consul default is 10 seconds. Similar to the interval this is a number with a V(s)
        or V(m) suffix to signify the units of seconds or minutes, for example V(15s) or V(1m). If no suffix is supplied V(s)
        is used by default, for example V(10) is equivalent to V(10s).
    type: str
  service_id:
    description:
      - The ID for the service, must be unique per node. If O(state=absent), defaults to the service name if supplied.
    type: str
"""

EXAMPLES = r"""
- name: Register tcp check for service 'nginx'
  community.general.consul_agent_check:
    name: nginx_tcp_check
    service_id: nginx
    interval: 60s
    tcp: localhost:80
    notes: "Nginx Check"

- name: Register http check for service 'nginx'
  community.general.consul_agent_check:
    name: nginx_http_check
    service_id: nginx
    interval: 60s
    http: http://localhost:80/status
    notes: "Nginx Check"

- name: Remove check for service 'nginx'
  community.general.consul_agent_check:
    state: absent
    id: nginx_http_check
    service_id: "{{ nginx_service.ID }}"
"""

RETURN = r"""
check:
  description: The check as returned by the Consul HTTP API.
  returned: always
  type: dict
  sample:
    CheckID: nginx_check
    ServiceID: nginx
    Interval: 30s
    Type: http
    Notes: Nginx Check
operation:
  description: The operation performed.
  returned: changed
  type: str
  sample: update
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._consul import (
    AUTH_ARGUMENTS_SPEC,
    OPERATION_CREATE,
    OPERATION_DELETE,
    OPERATION_READ,
    OPERATION_UPDATE,
    _ConsulModule,
    validate_check,
)

_ARGUMENT_SPEC = {
    "state": dict(default="present", choices=["present", "absent"]),
    "name": dict(type="str"),
    "id": dict(type="str"),
    "interval": dict(type="str"),
    "notes": dict(type="str"),
    "args": dict(type="list", elements="str"),
    "http": dict(type="str"),
    "tcp": dict(type="str"),
    "ttl": dict(type="str"),
    "timeout": dict(type="str"),
    "service_id": dict(type="str"),
}

_MUTUALLY_EXCLUSIVE = [
    ("args", "ttl", "tcp", "http"),
]

_REQUIRED_IF = [
    ("state", "present", ["name"]),
    ("state", "absent", ("id", "name"), True),
]

_REQUIRED_BY = {
    "args": "interval",
    "http": "interval",
    "tcp": "interval",
}

_ARGUMENT_SPEC.update(AUTH_ARGUMENTS_SPEC)


class ConsulAgentCheckModule(_ConsulModule):
    api_endpoint = "agent/check"
    result_key = "check"
    unique_identifiers = ["id", "name"]
    operational_attributes = {
        "Node",
        "CheckID",
        "Output",
        "ServiceName",
        "ServiceTags",
        "Status",
        "Type",
        "ExposedPort",
        "Definition",
    }

    def endpoint_url(self, operation, identifier=None):
        if operation == OPERATION_READ:
            return "agent/checks"
        if operation in [OPERATION_CREATE, OPERATION_UPDATE]:
            return f"{self.api_endpoint}/register"
        if operation == OPERATION_DELETE:
            return f"{self.api_endpoint}/deregister/{identifier}"

        return super().endpoint_url(operation, identifier)

    def read_object(self):
        url = self.endpoint_url(OPERATION_READ)
        checks = self.get(url)
        identifier = self.id_from_obj(self.params)
        if identifier in checks:
            return checks[identifier]
        return None

    def prepare_object(self, existing, obj):
        existing = super().prepare_object(existing, obj)
        validate_check(existing)
        return existing

    def delete_object(self, obj):
        if not self._module.check_mode:
            self.put(self.endpoint_url(OPERATION_DELETE, obj.get("CheckID")))
        return {}


def main():
    module = AnsibleModule(
        _ARGUMENT_SPEC,
        mutually_exclusive=_MUTUALLY_EXCLUSIVE,
        required_if=_REQUIRED_IF,
        required_by=_REQUIRED_BY,
        supports_check_mode=True,
    )

    consul_module = ConsulAgentCheckModule(module)
    consul_module.execute()


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2020 Haiyuan Zhang <haiyzhan@micosoft.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_openshiftmanagedclusterversion_info
version_added: '2.7.0'
short_description: Fetch available versions of Azure Red Hat OpenShift Managed Cluster
description:
    - fetch available version of Azure Red Hat OpenShift Managed Cluster instance.
options:
    location:
        description:
            - List install versions available for the defined region.
        required: true
        type: str
extends_documentation_fragment:
    - azure.azcollection.azure
author:
    - Maxim Babushkin (@maxbab)
'''

EXAMPLES = '''
- name: Obtain openshift versions for ARO cluster
  azure_rm_openshiftmanagedclusterversion_info:
    location: centralus
'''

RETURN = '''
versions:
    description:
        - openshift versions values
    returned: always
    type: list
'''

import json
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_rest import GenericRestClient


class Actions:
    NoAction, Create, Update, Delete = range(4)


class AzureRMOpenShiftManagedClustersVersionInfo(AzureRMModuleBaseExt):
    def __init__(self):
        self.module_arg_spec = dict(
            location=dict(
                type='str', required=True
            )
        )

        self.location = None

        self.results = dict(changed=False)
        self.mgmt_client = None
        self.state = None
        self.url = None
        self.status_code = [200]

        self.query_parameters = {}
        self.query_parameters['api-version'] = '2023-11-22'
        self.header_parameters = {}
        self.header_parameters['Content-Type'] = 'application/json; charset=utf-8'

        self.mgmt_client = None
        super(AzureRMOpenShiftManagedClustersVersionInfo, self).__init__(self.module_arg_spec, supports_check_mode=True, supports_tags=False)

    def exec_module(self, **kwargs):

        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        self.mgmt_client = self.get_mgmt_svc_client(GenericRestClient,
                                                    base_url=self._cloud_environment.endpoints.resource_manager)
        self.results = self.get_versions()
        return self.results

    def get_versions(self):
        response = None
        resp_results = {}
        results = {}
        # prepare url
        self.url = ('/subscriptions' +
                    '/{{ subscription_id }}' +
                    '/providers' +
                    '/Microsoft.RedHatOpenShift' +
                    '/locations' +
                    '/{{ location }}' +
                    '/openshiftversions')
        self.url = self.url.replace('{{ subscription_id }}', self.subscription_id)
        self.url = self.url.replace('{{ location }}', self.location)
        self.log("Fetch versions of openshift cluster.")
        try:
            response = self.mgmt_client.query(self.url,
                                              'GET',
                                              self.query_parameters,
                                              self.header_parameters,
                                              None,
                                              self.status_code,
                                              600,
                                              30)
            if isinstance(response.text, str):
                resp_results = json.loads(response.text)
            else:
                resp_results = json.loads(response.text())
        except Exception as e:
            self.log('Could not get info for @(Model.ModuleOperationNameUpper).')
        results['versions'] = self.format_versions(resp_results)
        return results

    def format_versions(self, version):
        result = list()
        if version.get('value'):
            for ver in version['value']:
                result.append(ver.get('properties').get('version'))
        return result


def main():
    AzureRMOpenShiftManagedClustersVersionInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python

# Copyright (c) 2018 Dario Zanzico (git@dariozanzico.com)
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: docker_stack
author: "Dario Zanzico (@dariko)"
short_description: docker stack module
description:
  - Manage docker stacks using the C(docker stack) command on the target node (see examples).
extends_documentation_fragment:
  - community.docker._docker.cli_documentation
  - community.docker._attributes
  - community.docker._attributes.actiongroup_docker
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none
  action_group:
    version_added: 3.6.0
  idempotent:
    support: full
options:
  name:
    description:
      - Stack name.
    type: str
    required: true
  state:
    description:
      - Service state.
    type: str
    default: "present"
    choices:
      - present
      - absent
  compose:
    description:
      - List of compose definitions. Any element may be a string referring to the path of the compose file on the target host
        or the YAML contents of a compose file nested as dictionary.
    type: list
    elements: raw
    default: []
  prune:
    description:
      - If true will add the C(--prune) option to the C(docker stack deploy) command. This will have docker remove the services
        not present in the current stack definition.
    type: bool
    default: false
  detach:
    description:
      - If V(false), the C(--detach=false) option is added to the C(docker stack deploy) command, allowing Docker to wait
        for tasks to converge before exiting.
      - If V(true) (default), Docker exits immediately instead of waiting for tasks to converge.
    type: bool
    default: true
    version_added: 4.1.0
  with_registry_auth:
    description:
      - If true will add the C(--with-registry-auth) option to the C(docker stack deploy) command. This will have docker send
        registry authentication details to Swarm agents.
    type: bool
    default: false
  resolve_image:
    description:
      - If set will add the C(--resolve-image) option to the C(docker stack deploy) command. This will have docker query the
        registry to resolve image digest and supported platforms. If not set, docker use "always" by default.
    type: str
    choices: ["always", "changed", "never"]
  absent_retries:
    description:
      - If larger than V(0) and O(state=absent) the module will retry up to O(absent_retries) times to delete the stack until
        all the resources have been effectively deleted. If the last try still reports the stack as not completely removed
        the module will fail.
    type: int
    default: 0
  absent_retries_interval:
    description:
      - Interval in seconds between consecutive O(absent_retries).
    type: int
    default: 1
  docker_cli:
    version_added: 3.6.0
  docker_host:
    version_added: 3.6.0
  tls_hostname:
    version_added: 3.6.0
  api_version:
    version_added: 3.6.0
  ca_path:
    version_added: 3.6.0
  client_cert:
    version_added: 3.6.0
  client_key:
    version_added: 3.6.0
  tls:
    version_added: 3.6.0
  validate_certs:
    version_added: 3.6.0
  cli_context:
    version_added: 3.6.0

requirements:
  - Docker CLI tool C(docker)
  - jsondiff
  - pyyaml
"""

RETURN = r"""
stack_spec_diff:
  description: |-
    Dictionary containing the differences between the 'Spec' field
    of the stack services before and after applying the new stack
    definition.
  sample: >
    "stack_spec_diff":
    {'test_stack_test_service': {'TaskTemplate': {'ContainerSpec': {delete: ['Env']}}}}
  returned: on change
  type: dict
"""

EXAMPLES = r"""
---
- name: Deploy stack from a compose file
  community.docker.docker_stack:
    state: present
    name: mystack
    compose:
      - /opt/docker-compose.yml

- name: Deploy stack from base compose file and override the web service
  community.docker.docker_stack:
    state: present
    name: mystack
    compose:
      - /opt/docker-compose.yml
      - version: '3'
        services:
          web:
            image: nginx:latest
            environment:
              ENVVAR: envvar

- name: Remove stack
  community.docker.docker_stack:
    name: mystack
    state: absent
"""


import json
import os
import tempfile
import traceback
import typing as t
from time import sleep

from ansible.module_utils.common.text.converters import to_text

from ansible_collections.community.docker.plugins.module_utils._common_cli import (
    AnsibleModuleDockerClient,
    DockerException,
)

try:
    from jsondiff import diff as json_diff

    HAS_JSONDIFF = True
except ImportError:
    HAS_JSONDIFF = False

try:
    from yaml import dump as yaml_dump

    HAS_YAML = True
except ImportError:
    HAS_YAML = False


def docker_stack_services(
    client: AnsibleModuleDockerClient, stack_name: str
) -> list[str]:
    dummy_rc, out, err = client.call_cli(
        "stack", "services", stack_name, "--format", "{{.Name}}"
    )
    if to_text(err) == f"Nothing found in stack: {stack_name}\n":
        return []
    return to_text(out).strip().split("\n")


def docker_service_inspect(
    client: AnsibleModuleDockerClient, service_name: str
) -> dict[str, t.Any] | None:
    rc, out, dummy_err = client.call_cli("service", "inspect", service_name)
    if rc != 0:
        return None
    ret = json.loads(out)[0]["Spec"]
    return ret


def docker_stack_deploy(
    client: AnsibleModuleDockerClient, stack_name: str, compose_files: list[str]
) -> tuple[int, str, str]:
    command = ["stack", "deploy"]
    if client.module.params["prune"]:
        command += ["--prune"]
    if not client.module.params["detach"]:
        command += ["--detach=false"]
    if client.module.params["with_registry_auth"]:
        command += ["--with-registry-auth"]
    if client.module.params["resolve_image"]:
        command += ["--resolve-image", client.module.params["resolve_image"]]
    for compose_file in compose_files:
        command += ["--compose-file", compose_file]
    command += [stack_name]
    rc, out, err = client.call_cli(*command)
    return rc, to_text(out), to_text(err)


def docker_stack_inspect(
    client: AnsibleModuleDockerClient, stack_name: str
) -> dict[str, dict[str, t.Any] | None]:
    ret: dict[str, dict[str, t.Any] | None] = {}
    for service_name in docker_stack_services(client, stack_name):
        ret[service_name] = docker_service_inspect(client, service_name)
    return ret


def docker_stack_rm(
    client: AnsibleModuleDockerClient,
    stack_name: str,
    retries: int,
    interval: int | float,
) -> tuple[int, str, str]:
    command = ["stack", "rm", stack_name]
    if not client.module.params["detach"]:
        command += ["--detach=false"]
    rc, out, err = client.call_cli(*command)

    while to_text(err) != f"Nothing found in stack: {stack_name}\n" and retries > 0:
        sleep(interval)
        retries = retries - 1
        rc, out, err = client.call_cli(*command)
    return rc, to_text(out), to_text(err)


def main() -> None:
    client = AnsibleModuleDockerClient(
        argument_spec={
            "name": {"type": "str", "required": True},
            "compose": {"type": "list", "elements": "raw", "default": []},
            "prune": {"type": "bool", "default": False},
            "detach": {"type": "bool", "default": True},
            "with_registry_auth": {"type": "bool", "default": False},
            "resolve_image": {"type": "str", "choices": ["always", "changed", "never"]},
            "state": {
                "type": "str",
                "default": "present",
                "choices": ["present", "absent"],
            },
            "absent_retries": {"type": "int", "default": 0},
            "absent_retries_interval": {"type": "int", "default": 1},
        },
        supports_check_mode=False,
    )

    if not HAS_JSONDIFF:
        client.fail("jsondiff is not installed, try 'pip install jsondiff'")

    if not HAS_YAML:
        client.fail("yaml is not installed, try 'pip install pyyaml'")

    try:
        state = client.module.params["state"]
        compose = client.module.params["compose"]
        name = client.module.params["name"]
        absent_retries = client.module.params["absent_retries"]
        absent_retries_interval = client.module.params["absent_retries_interval"]

        if state == "present":
            if not compose:
                client.fail(
                    "compose parameter must be a list containing at least one element"
                )

            compose_files = []
            for compose_def in compose:
                if isinstance(compose_def, dict):
                    compose_file_fd, compose_file = tempfile.mkstemp()
                    client.module.add_cleanup_file(compose_file)
                    with os.fdopen(compose_file_fd, "w") as stack_file:
                        compose_files.append(compose_file)
                        stack_file.write(yaml_dump(compose_def))
                elif isinstance(compose_def, str):
                    compose_files.append(compose_def)
                else:
                    client.fail(
                        f"compose element '{compose_def}' must be a string or a dictionary"
                    )

            before_stack_services = docker_stack_inspect(client, name)

            rc, out, err = docker_stack_deploy(client, name, compose_files)

            after_stack_services = docker_stack_inspect(client, name)

            if rc != 0:
                client.fail(
                    "docker stack up deploy command failed",
                    rc=rc,
                    stdout=out,
                    stderr=err,
                )

            before_after_differences = json_diff(
                before_stack_services, after_stack_services
            )
            for k in before_after_differences:
                if isinstance(before_after_differences[k], dict):
                    before_after_differences[k].pop("UpdatedAt", None)
                    before_after_differences[k].pop("Version", None)
                    if not list(before_after_differences[k].keys()):
                        before_after_differences.pop(k)

            if not before_after_differences:
                client.module.exit_json(
                    changed=False,
                    rc=rc,
                    stdout=out,
                    stderr=err,
                )
            else:
                client.module.exit_json(
                    changed=True,
                    rc=rc,
                    stdout=out,
                    stderr=err,
                    stack_spec_diff=json_diff(
                        before_stack_services,
                        after_stack_services,
                        dump=True,
                    ),
                )

        else:
            if docker_stack_services(client, name):
                rc, out, err = docker_stack_rm(
                    client, name, absent_retries, absent_retries_interval
                )
                if rc != 0:
                    client.module.fail_json(
                        msg="'docker stack down' command failed",
                        rc=rc,
                        stdout=out,
                        stderr=err,
                    )
                else:
                    client.module.exit_json(
                        changed=True,
                        msg=out,
                        rc=rc,
                        stdout=out,
                        stderr=err,
                    )
            client.module.exit_json(changed=False)
    except DockerException as e:
        client.fail(
            f"An unexpected Docker error occurred: {e}",
            exception=traceback.format_exc(),
        )


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_kvm_policy
short_description: Virtual KVM Policy configuration for Cisco Intersight
description:
  - Manages Virtual KVM Policy configuration on Cisco Intersight.
  - A policy to configure virtual KVM settings on Cisco Intersight managed servers.
  - When enabled, configures session limits, port settings, encryption, and tunneling options.
  - When disabled, only the enabled state is configured while other settings are ignored.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs/kvm/Policy/get/).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
      - Profiles, Policies, and Pools that are created within a Custom Organization are applicable only to devices in the same Organization.
    type: str
    default: default
  name:
    description:
      - The name assigned to the KVM Policy.
      - The name must be between 1 and 64 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  description:
    description:
      - The user-defined description for the KVM Policy.
      - Description can contain letters(a-z, A-Z), numbers(0-9), hyphen(-), period(.), colon(:), or an underscore(_).
    type: str
    aliases: [descr]
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
  enabled:
    description:
      - State of the vKVM service on the endpoint.
      - When set to false, all other configuration parameters are ignored and only the enabled state is configured.
      - When set to true, all other configuration parameters take effect.
    type: bool
    default: true
  maximum_sessions:
    description:
      - The maximum number of concurrent KVM sessions allowed.
      - Valid range is 1-4 sessions.
      - This parameter is ignored when C(enabled) is C(false).
    type: int
    default: 4
  remote_port:
    description:
      - The port used for KVM communication.
      - Valid range is 1024-65535.
      - This parameter is ignored when C(enabled) is C(false).
    type: int
    default: 2068
  enable_video_encryption:
    description:
      - If enabled, encrypts all video information sent through KVM.
      - Please note that this can no longer be disabled for servers running versions 4.2 and above.
      - This parameter is ignored when C(enabled) is C(false).
    type: bool
    default: true
  enable_local_server_video:
    description:
      - If enabled, displays KVM session on any monitor attached to the server.
      - This parameter is ignored when C(enabled) is C(false).
    type: bool
    default: true
  tunneled_kvm_enabled:
    description:
      - Enables Tunneled vKVM on the endpoint.
      - Applicable only for Device Connectors that support Tunneled vKVM.
      - This parameter is ignored when C(enabled) is C(false).
    type: bool
    default: false
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Create an enabled KVM Policy with default settings
  cisco.intersight.intersight_kvm_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "Default-KVM-Policy"
    description: "KVM policy with standard settings"
    enabled: true
    tags:
      - Key: "Environment"
        Value: "Production"

- name: Create a KVM Policy with custom settings
  cisco.intersight.intersight_kvm_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "Custom-KVM-Policy"
    description: "KVM policy with custom port and session limits"
    enabled: true
    maximum_sessions: 2
    remote_port: 3000
    enable_video_encryption: true
    enable_local_server_video: false
    tunneled_kvm_enabled: true
    tags:
      - Key: "Owner"
        Value: "DevOps"

- name: Create a disabled KVM Policy
  cisco.intersight.intersight_kvm_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "Disabled-KVM-Policy"
    description: "Disabled KVM policy for security compliance"
    enabled: false
    tags:
      - Key: "Security"
        Value: "Disabled"

- name: Delete a KVM Policy
  cisco.intersight.intersight_kvm_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "Old-KVM-Policy"
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Name": "Custom-KVM-Policy",
        "ObjectType": "kvm.Policy",
        "Enabled": true,
        "MaximumSessions": 2,
        "RemotePort": 3000,
        "EnableVideoEncryption": true,
        "EnableLocalServerVideo": false,
        "TunneledKvmEnabled": true,
        "Tags": [
            {
                "Key": "Owner",
                "Value": "DevOps"
            }
        ]
    }
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


def validate_kvm_policy_params(module):
    """
    Validate KVM policy parameters
    """
    # Validate maximum_sessions range
    max_sessions = module.params.get('maximum_sessions', 4)
    if max_sessions < 1 or max_sessions > 4:
        module.fail_json(msg="maximum_sessions must be between 1 and 4 (inclusive)")

    # Validate remote_port range
    remote_port = module.params.get('remote_port', 2068)
    if remote_port < 1024 or remote_port > 65535:
        module.fail_json(msg="remote_port must be between 1024 and 65535 (inclusive)")


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict'),
        enabled=dict(type='bool', default=True),
        maximum_sessions=dict(type='int', default=4),
        remote_port=dict(type='int', default=2068),
        enable_video_encryption=dict(type='bool', default=True),
        enable_local_server_video=dict(type='bool', default=True),
        tunneled_kvm_enabled=dict(type='bool', default=False)
    )
    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )

    # Validate KVM policy specific parameters
    validate_kvm_policy_params(module)

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    # Resource path used to configure policy
    resource_path = '/kvm/Policies'
    # Define API body used in compares or create
    intersight.api_body = {
        'Organization': {
            'Name': intersight.module.params['organization'],
        },
        'Name': intersight.module.params['name']
    }

    if module.params['state'] == 'present':
        intersight.set_tags_and_description()
        intersight.api_body.update({
            'Enabled': intersight.module.params['enabled']
        })
        # Only add KVM-specific settings if enabled is True
        if intersight.module.params['enabled']:
            intersight.api_body.update({
                'MaximumSessions': intersight.module.params['maximum_sessions'],
                'RemotePort': intersight.module.params['remote_port'],
                'EnableVideoEncryption': intersight.module.params['enable_video_encryption'],
                'EnableLocalServerVideo': intersight.module.params['enable_local_server_video'],
                'TunneledKvmEnabled': intersight.module.params['tunneled_kvm_enabled']
            })

    intersight.configure_policy_or_profile(resource_path=resource_path)

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_wireless_location_scanning_receivers
short_description: Resource module for organizations _wireless _location _scanning
  _receivers
description:
  - Manage operations create, update and delete of the resource organizations _wireless
    _location _scanning _receivers.
  - Add new receiver for scanning API.
  - Delete a scanning API receiver.
  - Change scanning API receiver settings.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  network:
    description: Add scanning API receiver for network.
    suboptions:
      id:
        description: Network ID.
        type: str
    type: dict
  organizationId:
    description: OrganizationId path parameter. Organization ID.
    type: str
  radio:
    description: Add scanning API Radio.
    suboptions:
      type:
        description: Radio Type whether WiFi or Bluetooth.
        type: str
    type: dict
  receiverId:
    description: ReceiverId path parameter. Receiver ID.
    type: str
  sharedSecret:
    description: Secret Value for Receiver.
    type: str
  url:
    description: Receiver Url.
    type: str
  version:
    description: Scanning API Version.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for wireless createOrganizationWirelessLocationScanningReceiver
    description: Complete reference of the createOrganizationWirelessLocationScanningReceiver
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!create-organization-wireless-location-scanning-receiver
  - name: Cisco Meraki documentation for wireless deleteOrganizationWirelessLocationScanningReceiver
    description: Complete reference of the deleteOrganizationWirelessLocationScanningReceiver
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!delete-organization-wireless-location-scanning-receiver
  - name: Cisco Meraki documentation for wireless updateOrganizationWirelessLocationScanningReceiver
    description: Complete reference of the updateOrganizationWirelessLocationScanningReceiver
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-organization-wireless-location-scanning-receiver
notes:
  - SDK Method used are
    wireless.Wireless.create_organization_wireless_location_scanning_receiver,
    wireless.Wireless.delete_organization_wireless_location_scanning_receiver,
    wireless.Wireless.update_organization_wireless_location_scanning_receiver,
  - Paths used are
    post /organizations/{organizationId}/wireless/location/scanning/receivers,
    delete /organizations/{organizationId}/wireless/location/scanning/receivers/{receiverId},
    put /organizations/{organizationId}/wireless/location/scanning/receivers/{receiverId},
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.organizations_wireless_location_scanning_receivers:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    network:
      id: L_1234
    organizationId: string
    radio:
      type: Wi-Fi
    sharedSecret: mysecretvalue
    url: https://www.myreceiver.com
    version: '3'
- name: Update by id
  cisco.meraki.organizations_wireless_location_scanning_receivers:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    organizationId: string
    radio:
      type: Wi-Fi
    receiverId: string
    url: https://www.myreceiver.com
    version: '3'
- name: Delete by id
  cisco.meraki.organizations_wireless_location_scanning_receivers:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: absent
    organizationId: string
    receiverId: string
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "network": {
        "id": "string",
        "name": "string"
      },
      "receiverId": "string",
      "url": "string",
      "version": "string",
      "radio": {
        "type": "string"
      }
    }
"""



#
# -*- coding: utf-8 -*-
# Copyright 2023 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

"""
The fact_diff plugin code
"""
from __future__ import absolute_import, division, print_function

import difflib
import re

from collections.abc import MutableMapping

from ansible.plugins.callback import CallbackBase


__metaclass__ = type


from ansible.errors import AnsibleFilterError


def _raise_error(msg):
    """Raise an error message, prepend with filter name
    :param msg: The message
    :type msg: str
    :raises: AnsibleError
    """
    error = "Error when using filter plugin 'fact_diff': {msg}".format(msg=msg)
    raise AnsibleFilterError(error)


def fact_diff(before, after, plugin, common):
    """Compare two facts or variables and get a diff.
    :param before: The first fact to be used in the comparison.
    :type before: raw
    :param after: The second fact to be used in the comparison.
    :type after: raw
    :param plugin: The name of the plugin in collection format
    :type plugin: string
    """
    if plugin.get("name") == "ansible.utils.native":
        result = fact_diff_native().run_diff(before, after, plugin, common)
    return result


class fact_diff_native(CallbackBase):
    def _check_valid_regexes(self, skip_lines):
        if skip_lines:
            for idx, regex in enumerate(skip_lines):
                try:
                    skip_lines[idx] = re.compile(regex)
                except re.error as exc:
                    msg = "The regex '{regex}', is not valid. The error was {err}.".format(
                        regex=regex,
                        err=str(exc),
                    )
                    _raise_error(msg)

    def _xform(self, before, after, skip_lines):
        if skip_lines:
            if isinstance(before, str):
                before = before.splitlines()
            if isinstance(after, str):
                after = after.splitlines()
            before = [
                line for line in before if not any(regex.match(str(line)) for regex in skip_lines)
            ]
            after = [
                line for line in after if not any(regex.match(str(line)) for regex in skip_lines)
            ]
        if isinstance(before, list):
            before = "\n".join(map(str, before)) + "\n"
        if isinstance(after, list):
            after = "\n".join(map(str, after)) + "\n"
        return before, after, skip_lines

    def get_fact_diff(self, difflist):
        if not isinstance(difflist, list):
            difflist = [difflist]
        ret = []
        for diff in difflist:
            if "before" in diff and "after" in diff:
                # format complex structures into 'files'
                for x in ["before", "after"]:
                    if isinstance(diff[x], MutableMapping):
                        diff[x] = self._serialize_diff(diff[x])
                    elif diff[x] is None:
                        diff[x] = ""
                if "before_header" in diff:
                    before_header = "before: %s" % diff["before_header"]
                else:
                    before_header = "before"
                if "after_header" in diff:
                    after_header = "after: %s" % diff["after_header"]
                else:
                    after_header = "after"
                before_lines = diff["before"].splitlines(True)
                after_lines = diff["after"].splitlines(True)
                if before_lines and not before_lines[-1].endswith("\n"):
                    before_lines[-1] += "\n\\ No newline at end of file\n"
                if after_lines and not after_lines[-1].endswith("\n"):
                    after_lines[-1] += "\n\\ No newline at end of file\n"
                diff_context = (
                    len(before_lines) if len(before_lines) > len(after_lines) else len(after_lines)
                )
                differ = difflib.unified_diff(
                    before_lines,
                    after_lines,
                    fromfile=before_header,
                    tofile=after_header,
                    fromfiledate="",
                    tofiledate="",
                    n=diff_context,
                )
                difflines = list(differ)
                has_diff = False
                for line in difflines:
                    has_diff = True
                    if diff["common"]:
                        if line.startswith("+") or line.startswith("-"):
                            pass
                        else:
                            ret.append(line)
                    else:
                        ret.append(line)
                if has_diff:
                    ret.append("\n")
            if "prepared" in diff:
                ret.append(diff["prepared"])
        return "".join(ret)

    def run_diff(self, before, after, plugin, common):
        skip_lines = plugin["vars"].get("skip_lines")
        self._check_valid_regexes(skip_lines=skip_lines)
        before, after, skip_lines = self._xform(before, after, skip_lines=skip_lines)
        diff = self.get_fact_diff({"before": before, "after": after, "common": common})
        ansi_escape = re.compile(r"\x1B[@-_][0-?]*[ -/]*[@-~]")
        diff_text = ansi_escape.sub("", diff)
        result = list(diff_text.splitlines())
        return result



#!/usr/bin/python
# Copyright (c) 2017, Loic Blot <loic.blot@unix-experience.fr>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: logstash_plugin
short_description: Manage Logstash plugins
description:
  - Manages Logstash plugins.
author: Loic Blot (@nerzhul)
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  name:
    type: str
    description:
      - Install plugin with that name.
    required: true
  state:
    type: str
    description:
      - Apply plugin state.
    choices: ["present", "absent"]
    default: present
  plugin_bin:
    type: path
    description:
      - Specify logstash-plugin to use for plugin management.
    default: /usr/share/logstash/bin/logstash-plugin
  proxy_host:
    type: str
    description:
      - Proxy host to use during plugin installation.
      - Can be specified as a hostname (for example, V(myproxy.example.com)) or as a URL (for example, V(http://myproxy.example.com)).
        When specified without a scheme, V(http://) is assumed.
      - Sets the O(proxy_host):O(proxy_port) combination as the E(http_proxy) and E(https_proxy) environment variables
        when running the C(logstash-plugin) command.
  proxy_port:
    type: str
    description:
      - Proxy port to use during plugin installation.
  version:
    type: str
    description:
      - Specify version of the plugin to install. If the plugin exists with a previous version, it is B(not) updated.
"""

EXAMPLES = r"""
- name: Install Logstash beats input plugin
  community.general.logstash_plugin:
    state: present
    name: logstash-input-beats

- name: Install specific version of a plugin
  community.general.logstash_plugin:
    state: present
    name: logstash-input-syslog
    version: '3.2.0'

- name: Uninstall Logstash plugin
  community.general.logstash_plugin:
    state: absent
    name: logstash-filter-multiline

- name: Install Logstash plugin with alternate heap size
  community.general.logstash_plugin:
    state: present
    name: logstash-input-beats
  environment:
    LS_JAVA_OPTS: "-Xms256m -Xmx256m"
"""

from ansible.module_utils.basic import AnsibleModule

PACKAGE_STATE_MAP = dict(present="install", absent="remove")


def is_plugin_present(module, plugin_bin, plugin_name):
    cmd_args = [plugin_bin, "list", plugin_name]
    rc, out, err = module.run_command(cmd_args)
    return rc == 0


def parse_error(string):
    reason = "reason: "
    try:
        return string[string.index(reason) + len(reason) :].strip()
    except ValueError:
        return string


def install_plugin(module, plugin_bin, plugin_name, version, proxy_host, proxy_port):
    cmd_args = [plugin_bin, PACKAGE_STATE_MAP["present"]]

    if version:
        cmd_args.extend(["--version", version])

    cmd_args.append(plugin_name)

    environ_update = {}
    if proxy_host and proxy_port:
        scheme = proxy_host if "://" in proxy_host else f"http://{proxy_host}"
        proxy_url = f"{scheme}:{proxy_port}"
        environ_update = {"http_proxy": proxy_url, "https_proxy": proxy_url}

    cmd = " ".join(cmd_args)

    if module.check_mode:
        rc, out, err = 0, "check mode", ""
    else:
        rc, out, err = module.run_command(cmd_args, environ_update=environ_update)

    if rc != 0:
        reason = parse_error(out)
        module.fail_json(msg=reason, stderr=err)

    return True, cmd, out, err


def remove_plugin(module, plugin_bin, plugin_name):
    cmd_args = [plugin_bin, PACKAGE_STATE_MAP["absent"], plugin_name]

    cmd = " ".join(cmd_args)

    if module.check_mode:
        rc, out, err = 0, "check mode", ""
    else:
        rc, out, err = module.run_command(cmd_args)

    if rc != 0:
        reason = parse_error(out)
        module.fail_json(msg=reason, stderr=err)

    return True, cmd, out, err


def main():
    module = AnsibleModule(
        argument_spec=dict(
            name=dict(required=True),
            state=dict(default="present", choices=list(PACKAGE_STATE_MAP.keys())),
            plugin_bin=dict(default="/usr/share/logstash/bin/logstash-plugin", type="path"),
            proxy_host=dict(),
            proxy_port=dict(),
            version=dict(),
        ),
        supports_check_mode=True,
    )
    module.run_command_environ_update = {"LANGUAGE": "C", "LC_ALL": "C"}

    name = module.params["name"]
    state = module.params["state"]
    plugin_bin = module.params["plugin_bin"]
    proxy_host = module.params["proxy_host"]
    proxy_port = module.params["proxy_port"]
    version = module.params["version"]

    present = is_plugin_present(module, plugin_bin, name)

    # skip if the state is correct
    if (present and state == "present") or (state == "absent" and not present):
        module.exit_json(changed=False, name=name, state=state)

    if state == "present":
        changed, cmd, out, err = install_plugin(module, plugin_bin, name, version, proxy_host, proxy_port)
    elif state == "absent":
        changed, cmd, out, err = remove_plugin(module, plugin_bin, name)

    module.exit_json(changed=changed, cmd=cmd, name=name, state=state, stdout=out, stderr=err)


if __name__ == "__main__":
    main()



# This code is part of the Ansible collection community.docker, but is an independent component.
# This particular file, and this file only, is based on the Docker SDK for Python (https://github.com/docker/docker-py/)
#
# Copyright (c) 2016-2022 Docker, Inc.
#
# It is licensed under the Apache 2.0 license (see LICENSES/Apache-2.0.txt in this collection)
# SPDX-License-Identifier: Apache-2.0

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import typing as t

from .utils import format_environment


class ProxyConfig(dict):
    """
    Hold the client's proxy configuration
    """

    @property
    def http(self) -> str | None:
        return self.get("http")

    @property
    def https(self) -> str | None:
        return self.get("https")

    @property
    def ftp(self) -> str | None:
        return self.get("ftp")

    @property
    def no_proxy(self) -> str | None:
        return self.get("no_proxy")

    @staticmethod
    def from_dict(config: dict[str, str]) -> ProxyConfig:
        """
        Instantiate a new ProxyConfig from a dictionary that represents a
        client configuration, as described in `the documentation`_.

        .. _the documentation:
            https://docs.docker.com/network/proxy/#configure-the-docker-client
        """
        return ProxyConfig(
            http=config.get("httpProxy"),
            https=config.get("httpsProxy"),
            ftp=config.get("ftpProxy"),
            no_proxy=config.get("noProxy"),
        )

    def get_environment(self) -> dict[str, str]:
        """
        Return a dictionary representing the environment variables used to
        set the proxy settings.
        """
        env = {}
        if self.http:
            env["http_proxy"] = env["HTTP_PROXY"] = self.http
        if self.https:
            env["https_proxy"] = env["HTTPS_PROXY"] = self.https
        if self.ftp:
            env["ftp_proxy"] = env["FTP_PROXY"] = self.ftp
        if self.no_proxy:
            env["no_proxy"] = env["NO_PROXY"] = self.no_proxy
        return env

    @t.overload
    def inject_proxy_environment(self, environment: list[str]) -> list[str]: ...

    @t.overload
    def inject_proxy_environment(
        self, environment: list[str] | None
    ) -> list[str] | None: ...

    def inject_proxy_environment(
        self, environment: list[str] | None
    ) -> list[str] | None:
        """
        Given a list of strings representing environment variables, prepend the
        environment variables corresponding to the proxy settings.
        """
        if not self:
            return environment

        proxy_env = format_environment(self.get_environment())
        if not environment:
            return proxy_env
        # It is important to prepend our variables, because we want the
        # variables defined in "environment" to take precedence.
        return proxy_env + environment

    def __str__(self) -> str:
        return f"ProxyConfig(http={self.http}, https={self.https}, ftp={self.ftp}, no_proxy={self.no_proxy})"



# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
"""
The eos_lldp_interfaces class
It is in this file where the current configuration (as dict)
is compared to the provided configuration (as dict) and the command set
necessary to bring the current configuration to it's desired end-state is
created
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.cfg.base import (
    ConfigBase,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    dict_diff,
    param_list_to_dict,
    to_list,
)

from ansible_collections.arista.eos.plugins.module_utils.network.eos.facts.facts import Facts
from ansible_collections.arista.eos.plugins.module_utils.network.eos.utils.utils import (
    normalize_interface,
)


class Lldp_interfaces(ConfigBase):
    """
    The eos_lldp_interfaces class
    """

    gather_subset = ["!all", "!min"]

    gather_network_resources = ["lldp_interfaces"]

    def get_lldp_interfaces_facts(self, data=None):
        """Get the 'facts' (the current configuration)

        :rtype: A dictionary
        :returns: The current configuration as a dictionary
        """
        facts, _warnings = Facts(self._module).get_facts(
            self.gather_subset,
            self.gather_network_resources,
            data=data,
        )
        lldp_interfaces_facts = facts["ansible_network_resources"].get(
            "lldp_interfaces",
        )
        if not lldp_interfaces_facts:
            return []
        return lldp_interfaces_facts

    def execute_module(self):
        """Execute the module

        :rtype: A dictionary
        :returns: The result from module execution
        """
        result = {"changed": False}
        warnings = list()
        commands = list()

        if self.state in self.ACTION_STATES:
            existing_lldp_interfaces_facts = self.get_lldp_interfaces_facts()
        else:
            existing_lldp_interfaces_facts = []

        if self.state in self.ACTION_STATES or self.state == "rendered":
            commands.extend(self.set_config(existing_lldp_interfaces_facts))

        if commands and self.state in self.ACTION_STATES:
            if not self._module.check_mode:
                self._connection.edit_config(commands)
            result["changed"] = True
        if self.state in self.ACTION_STATES:
            result["commands"] = commands

        if self.state in self.ACTION_STATES or self.state == "gathered":
            changed_lldp_interfaces_facts = self.get_lldp_interfaces_facts()
        elif self.state == "rendered":
            result["rendered"] = commands
        elif self.state == "parsed":
            running_config = self._module.params["running_config"]
            if not running_config:
                self._module.fail_json(
                    msg="value of running_config parameter must not be empty for state parsed",
                )
            result["parsed"] = self.get_lldp_interfaces_facts(
                data=running_config,
            )

        if self.state in self.ACTION_STATES:
            result["before"] = existing_lldp_interfaces_facts
            if result["changed"]:
                result["after"] = changed_lldp_interfaces_facts

        elif self.state == "gathered":
            result["gathered"] = changed_lldp_interfaces_facts

        result["warnings"] = warnings
        return result

    def set_config(self, existing_lldp_interfaces_facts):
        """Collect the configuration from the args passed to the module,
            collect the current configuration (as a dict from facts)

        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        want = self._module.params["config"]
        have = existing_lldp_interfaces_facts
        resp = self.set_state(want, have)
        return to_list(resp)

    def set_state(self, want, have):
        """Select the appropriate function based on the state provided

        :param want: the desired configuration as a dictionary
        :param have: the current configuration as a dictionary
        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        state = self._module.params["state"]
        if state in ("merged", "replaced", "overridden", "rendered") and not want:
            self._module.fail_json(
                msg="value of config parameter must not be empty for state {0}".format(
                    state,
                ),
            )
        want = param_list_to_dict(want, remove_key=False)
        have = param_list_to_dict(have, remove_key=False)
        if state == "overridden":
            commands = self._state_overridden(want, have)
        elif state == "deleted":
            commands = self._state_deleted(want, have)
        elif state == "merged" or state == "rendered":
            commands = self._state_merged(want, have)
        elif state == "replaced":
            commands = self._state_replaced(want, have)
        return commands

    @staticmethod
    def _state_replaced(want, have):
        """The command generator when state is replaced

        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        commands = []
        for key, desired in want.items():
            interface_name = normalize_interface(key)
            if interface_name in have:
                extant = have[interface_name]
            else:
                extant = dict(name=interface_name)

            add_config = dict_diff(extant, desired)
            del_config = dict_diff(desired, extant)

            commands.extend(
                generate_commands(interface_name, add_config, del_config),
            )

        return commands

    @staticmethod
    def _state_overridden(want, have):
        """The command generator when state is overridden

        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        commands = []
        for key, extant in have.items():
            if key in want:
                desired = want[key]
            else:
                desired = dict(name=key)

            add_config = dict_diff(extant, desired)
            del_config = dict_diff(desired, extant)

            commands.extend(generate_commands(key, add_config, del_config))

        return commands

    @staticmethod
    def _state_merged(want, have):
        """The command generator when state is merged

        :rtype: A list
        :returns: the commands necessary to merge the provided into
                  the current configuration
        """
        commands = []
        for key, desired in want.items():
            interface_name = normalize_interface(key)
            if interface_name in have:
                extant = have[interface_name]
            else:
                extant = dict(name=interface_name)

            add_config = dict_diff(extant, desired)

            commands.extend(generate_commands(interface_name, add_config, {}))

        return commands

    @staticmethod
    def _state_deleted(want, have):
        """The command generator when state is deleted

        :rtype: A list
        :returns: the commands necessary to remove the current configuration
                  of the provided objects
        """
        commands = []
        for key in want.keys():
            interface_name = normalize_interface(key)
            desired = dict(name=interface_name)
            if interface_name in have:
                extant = have[interface_name]
            else:
                continue

            del_config = dict_diff(desired, extant)

            commands.extend(generate_commands(interface_name, {}, del_config))

        return commands


def generate_commands(name, to_set, to_remove):
    commands = []
    for key, value in to_set.items():
        if value is None:
            continue

        prefix = "" if value else "no "
        commands.append("{0}lldp {1}".format(prefix, key))

    for key in to_remove:
        commands.append("lldp {0}".format(key))

    if commands:
        commands.insert(0, "interface {0}".format(name))

    return commands



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: elb_classic_lb_info
version_added: 1.0.0
version_added_collection: community.aws
short_description: Gather information about EC2 Classic Elastic Load Balancers in AWS
description:
  - Gather information about EC2 Classic Elastic Load Balancers in AWS.
author:
  - "Michael Schultz (@mjschultz)"
  - "Fernando Jose Pando (@nand0p)"
options:
  names:
    description:
      - List of ELB names to gather information about. Pass this option to gather information about a set of ELBs, otherwise, all ELBs are returned.
    type: list
    elements: str
    default: []
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.
# Output format tries to match amazon.aws.elb_classic_lb module input parameters

- name: Gather information about all ELBs
  amazon.aws.elb_classic_lb_info:
  register: elb_info

- ansible.builtin.debug:
    msg: "{{ item.dns_name }}"
  loop: "{{ elb_info.elbs }}"

- name: Gather information about a particular ELB
  amazon.aws.elb_classic_lb_info:
    names: frontend-prod-elb
  register: elb_info

- ansible.builtin.debug:
    msg: "{{ elb_info.elbs.0.dns_name }}"

- name: Gather information about a set of ELBs
  amazon.aws.elb_classic_lb_info:
    names:
      - frontend-prod-elb
      - backend-prod-elb
  register: elb_info

- ansible.builtin.debug:
    msg: "{{ item.dns_name }}"
  loop: "{{ elb_info.elbs }}"
"""

RETURN = r"""
elbs:
  description: A list of load balancers.
  returned: always
  type: list
  elements: dict
  contains:
    attributes:
      description: Information about the load balancer attributes.
      returned: always
      type: dict
      contains:
        access_log:
          description: Information on whether access logs are enabled or not.
          type: dict
          sample: {
                    "enabled": false
                  }
        additional_attributes:
          description: Information about additional load balancer attributes.
          type: list
          elements: dict
          sample: [
                    {
                        "key": "elb.http.desyncmitigationmode",
                        "value": "defensive"
                    }
                  ]
        connection_draining:
          description:
            - Information on connection draining configuration of elastic load balancer.
          type: dict
          sample: {
                    "enabled": true,
                    "timeout": 300
                  }
          contains:
            enabled:
              description: Whether connection draining is enabled.
              type: bool
              returned: always
            timeout:
              description: The maximum time, in seconds, to keep the existing connections open before deregistering the instances.
              type: int
              returned: always
        connection_settings:
          description: Information on connection settings.
          type: dict
          sample: {
                    "idle_timeout": 60
                  }
        cross_zone_load_balancing:
          description: Information on whether cross zone load balancing is enabled or not.
          type: dict
          sample: {
                    "enabled": true
                  }
    availability_zones:
      description: The Availability Zones for the load balancer.
      type: list
      elements: str
      returned: always
      sample: [
                "us-west-2a"
              ]
    backend_server_descriptions:
      description: Information about your EC2 instances.
      type: list
      elements: dict
      returned: always
      sample: [
                {
                    instance_port: 8085,
                    policy_names: [
                        'MyPolicy1',
                    ]
                },
              ]
    canonical_hosted_zone_name:
      description: The DNS name of the load balancer.
      type: str
      returned: always
      sample: "test-123456789.us-west-2.elb.amazonaws.com"
    canonical_hosted_zone_name_id:
      description: The ID of the Amazon Route 53 hosted zone for the load balancer.
      type: str
      returned: always
      sample: "Z1Z1ZZ5HABSF5"
    created_time:
      description: The date and time the load balancer was created.
      type: str
      returned: always
      sample: "2024-09-04T17:52:22.270000+00:00"
    dns_name:
      description: The DNS name of the load balancer.
      type: str
      returned: "always"
      sample: "test-123456789.us-west-2.elb.amazonaws.com"
    health_check:
      description: Information about the health checks conducted on the load balancer.
      type: dict
      returned: always
      sample: {
                "healthy_threshold": 10,
                "interval": 5,
                "target": "HTTP:80/index.html",
                "timeout": 2,
                "unhealthy_threshold": 2
              }
      contains:
        healthy_threshold:
          description: The number of consecutive health checks successes required before moving the instance to the Healthy state.
          type: int
          returned: always
        interval:
          description: The approximate interval, in seconds, between health checks of an individual instance.
          type: int
          returned: always
        target:
          description: The instance being checked. The protocol is either TCP, HTTP, HTTPS, or SSL. The range of valid ports is one (1) through 65535.
          type: str
          returned: always
        timeout:
          description: The amount of time, in seconds, during which no response means a failed health check.
          type: int
          returned: always
        unhealthy_threshold:
          description: The number of consecutive health checks successes required before moving the instance to the Unhealthy state.
          type: int
          returned: always
    instances:
      description: The IDs of the instances for the load balancer.
      type: list
      elements: dict
      returned: always
      sample: [
                {
                    "instance_id": "i-11d1f111ea111111b"
                }
              ]
    instances_inservice:
      description: Information about instances for load balancer in state "InService".
      type: list
      returned: always
      sample: [
                "i-11d1f111ea111111b"
              ]
    instances_inservice_count:
      description: Total number of instances for load balancer with state "InService".
      type: int
      returned: always
      sample: 1
    instances_outofservice:
      description: Information about instances for load balancer in state "OutOfService".
      type: list
      returned: always
      sample: [
                "i-11d1f111ea111111b"
              ]
    instances_outofservice_count:
      description: Total number of instances for load balancer with state "OutOfService".
      type: int
      returned: always
      sample: 0
    instances_unknownservice:
      description: Information about instances for load balancer in state "Unknown".
      type: list
      returned: always
      sample: [
                "i-11d1f111ea111111b"
              ]
    instances_unknownservice_count:
      description: Total number of instances for load balancer with state "Unknown".
      type: int
      returned: always
      sample: 1
    listener_descriptions:
      description: Information about the listeners for the load balancer.
      type: list
      elements: dict
      returned: always
      sample: [
                {
                  "listener": {
                      "instance_port": 80,
                      "instance_protocol": "HTTP",
                      "load_balancer_port": 80,
                      "protocol": "HTTP"
                  },
                  "policy_names": []
                }
              ]
    load_balancer_name:
      description: The name of the elastic load balancer.
      type: str
      returned: always
      sample: "MyLoadBalancer"
    policies:
      description: Information about the policies defined for the load balancer.
      type: dict
      returned: always
      sample: {
                "app_cookie_stickiness_policies": [],
                "lb_cookie_stickiness_policies": [],
                "other_policies": []
              }
      contains:
        app_cookie_stickiness_policies:
          description: The stickiness policies created using CreateAppCookieStickinessPolicy.
          type: list
          returned: always
        lb_cookie_stickiness_policies:
          description: The stickiness policies created using CreateLBCookieStickinessPolicy.
          type: list
          returned: always
        other_policies:
          description: The policies other than the stickiness policies.
          type: list
          returned: always
    scheme:
      description: The type of load balancer.
      type: str
      returned: always
      sample: "internet-facing"
    security_groups:
      description: The security groups for the load balancer.
      type: list
      returned: always
      sample: [
                "sg-111111af1111cb111"
              ]
    source_security_group:
      description:
        - The security group for the load balancer,
          which are used as part of inbound rules for registered instances.
      type: dict
      returned: always
      sample: {
                  "group_name": "default",
                  "owner_alias": "721111111111"
              }
      contains:
        group_name:
          description: The name of the security group.
          type: str
          returned: always
        owner_alias:
          description: The owner of the security group.
          type: str
          returned: always
    subnets:
      description: The IDs of the subnets for the load balancer.
      type: list
      returned: always
      sample: [
                "subnet-111111af1111cb111"
              ]
    tags:
      description: The tags associated with a load balancer.
      type: dict
      returned: always
      sample: {
                "Env": "Dev",
                "Owner": "Dev001"
              }
    vpc_id:
      description: The ID of the VPC for the load balancer.
      type: str
      returned: always
      sample: "vpc-0cc28c9e20d111111"
"""

from typing import Any
from typing import Dict
from typing import List
from typing import Tuple
from typing import Union

try:
    import botocore
except ImportError:
    pass  # caught by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.botocore import is_boto3_error_code
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict


def list_elbs(connection: Any, load_balancer_names: List[str]) -> List[Dict]:
    """
    List Elastic Load Balancers (ELBs) and their detailed information.

    Parameters:
      connection (boto3.client): The Boto3 ELB client object.
      load_balancer_names (List[str]): List of ELB names to gather information about.

    Returns:
      A list of dictionaries where each dictionary contains informtion about one ELB.
    """
    results = []

    if not load_balancer_names:
        for lb in get_all_lb(connection):
            results.append(describe_elb(connection, lb))

    for load_balancer_name in load_balancer_names:
        lb = get_lb(connection, load_balancer_name)
        if not lb:
            continue
        results.append(describe_elb(connection, lb))
    return results


def describe_elb(connection: Any, lb: Dict[str, Any]) -> Dict[str, Any]:
    """
    Describes an Elastic Load Balancer (ELB).

    Parameters:
      connection (boto3.client): The Boto3 ELB client object.
      lb (Dict): Dictionary containing ELB .

    Returns:
      A dictionary with detailed information of the ELB.
    """
    description = camel_dict_to_snake_dict(lb)
    name = lb["LoadBalancerName"]
    instances = lb.get("Instances", [])
    description["tags"] = get_tags(connection, name)
    description["instances_inservice"], description["instances_inservice_count"] = lb_instance_health(
        connection, name, instances, "InService"
    )
    description["instances_outofservice"], description["instances_outofservice_count"] = lb_instance_health(
        connection, name, instances, "OutOfService"
    )
    description["instances_unknownservice"], description["instances_unknownservice_count"] = lb_instance_health(
        connection, name, instances, "Unknown"
    )
    description["attributes"] = get_lb_attributes(connection, name)
    return description


@AWSRetry.jittered_backoff()
def get_all_lb(connection: Any) -> List:
    """
    Get paginated result for information of all Elastic Load Balancers.

    Parameters:
        connection (boto3.client): The Boto3 ELB client object.

    Returns:
        A list of dictionaries containing descriptions of all ELBs.
    """
    paginator = connection.get_paginator("describe_load_balancers")
    return paginator.paginate().build_full_result()["LoadBalancerDescriptions"]


def get_lb(connection: Any, load_balancer_name: str) -> Union[Dict[str, Any], List]:
    """
    Describes a specific Elastic Load Balancer (ELB) by name.

    Parameters:
      connection (boto3.client): The Boto3 ELB client object.
      load_balancer_name (str): Name of the ELB to gather information about.

    Returns:
      A dictionary with detailed information of the specified ELB.
    """
    try:
        return connection.describe_load_balancers(aws_retry=True, LoadBalancerNames=[load_balancer_name])[
            "LoadBalancerDescriptions"
        ][0]
    except is_boto3_error_code("LoadBalancerNotFound"):
        return []


def get_lb_attributes(connection: Any, load_balancer_name: str) -> Dict[str, Any]:
    """
    Retrieves attributes of specific Elastic Load Balancer (ELB) by name.

    Parameters:
      connection (boto3.client): The Boto3 ELB client object.
      load_balancer_name (str): Name of the ELB to gather information about.

    Returns:
      A dictionary with detailed information of the attributes of specified ELB.
    """
    attributes = connection.describe_load_balancer_attributes(aws_retry=True, LoadBalancerName=load_balancer_name).get(
        "LoadBalancerAttributes", {}
    )
    return camel_dict_to_snake_dict(attributes)


def get_tags(connection: Any, load_balancer_name: str) -> Dict[str, Any]:
    """
    Retrieves tags of specific Elastic Load Balancer (ELB) by name.

    Parameters:
      connection (boto3.client): The Boto3 ELB client object.
      load_balancer_name (str): Name of the ELB to gather information about.

    Returns:
      A dictionary of tags associated with the specified ELB.
    """
    tags = connection.describe_tags(aws_retry=True, LoadBalancerNames=[load_balancer_name])["TagDescriptions"]
    if not tags:
        return {}
    return boto3_tag_list_to_ansible_dict(tags[0]["Tags"])


def lb_instance_health(
    connection: Any, load_balancer_name: str, instances: List[Dict[str, Any]], state: str
) -> Tuple[List[str], int]:
    """
    Describes the health status of instances associated with a specific Elastic Load Balancer (ELB).

    Parameters:
        connection (Any): The Boto3 client object for ELB.
        load_balancer_name (str): The name of the ELB.
        instances (List[Dict]): List of dictionaries containing instances associated with the ELB.
        state (str): The health state to filter by (e.g., "InService", "OutOfService", "Unknown").

    Returns:
        Tuple[List, int]: A tuple containing a list of instance IDs matching state and the count of matching instances.
    """
    instance_states = connection.describe_instance_health(LoadBalancerName=load_balancer_name, Instances=instances).get(
        "InstanceStates", []
    )
    instate = [instance["InstanceId"] for instance in instance_states if instance["State"] == state]
    return instate, len(instate)


def main():
    argument_spec = dict(
        names=dict(default=[], type="list", elements="str"),
    )
    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    connection = module.client("elb", retry_decorator=AWSRetry.jittered_backoff(retries=5, delay=5))

    try:
        elbs = list_elbs(connection, module.params.get("names"))
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to get load balancer information.")

    module.exit_json(elbs=elbs)


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2024, Philippe Duveau <pduvax@gmail.com>
# Copyright (c) 2019, Maciej Delmanowski <drybjed@gmail.com> (ldap_attrs.py)
# Copyright (c) 2017, Alexander Korinek <noles@a3k.net> (ldap_attrs.py)
# Copyright (c) 2016, Peter Sagerson <psagers@ignorare.net> (ldap_attrs.py)
# Copyright (c) 2016, Jiri Tyr <jiri.tyr@gmail.com> (ldap_attrs.py)
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# The code of this module is derived from that of ldap_attrs.py

from __future__ import annotations

DOCUMENTATION = r"""
module: ldap_inc
short_description: Use the Modify-Increment LDAP V3 feature to increment an attribute value
version_added: 10.2.0
description:
  - Atomically increments the value of an attribute and return its new value.
notes:
  - When implemented by the directory server, the module uses the ModifyIncrement extension defined in L(RFC4525, https://www.rfc-editor.org/rfc/rfc4525.html)
    and the control PostRead. This extension and the control are implemented in OpenLdap but not all directory servers implement
    them. In this case, the module automatically uses a more classic method based on two phases, first the current value is
    read then the modify operation remove the old value and add the new one in a single request. If the value has changed
    by a concurrent call then the remove action fails. Then the sequence is retried 3 times before raising an error to the
    playbook. In an heavy modification environment, the module does not guarante to be systematically successful.
  - This only deals with integer attribute of an existing entry. To modify attributes of an entry, see M(community.general.ldap_attrs)
    or to add or remove whole entries, see M(community.general.ldap_entry).
author:
  - Philippe Duveau (@pduveau)
requirements:
  - python-ldap
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  dn:
    required: true
    type: str
    description:
      - The DN entry containing the attribute to increment.
  attribute:
    required: true
    type: str
    description:
      - The attribute to increment.
  increment:
    type: int
    default: 1
    description:
      - The value of the increment to apply.
  method:
    type: str
    default: auto
    choices: [auto, rfc4525, legacy]
    description:
      - If V(auto), the module determines automatically the method to use.
      - If V(rfc4525) or V(legacy) force to use the corresponding method.
extends_documentation_fragment:
  - community.general._ldap.documentation
  - community.general._attributes
"""


EXAMPLES = r"""
- name: Increments uidNumber 1 Number for example.com
  community.general.ldap_inc:
    dn: "cn=uidNext,ou=unix-management,dc=example,dc=com"
    attribute: "uidNumber"
    increment: "1"
  register: ldap_uidNumber_sequence

- name: Modifies the user to define its identification number (uidNumber) when incrementation is successful
  community.general.ldap_attrs:
    dn: "cn=john,ou=posix-users,dc=example,dc=com"
    state: present
    attributes:
      - uidNumber: "{{ ldap_uidNumber_sequence.value }}"
  when: ldap_uidNumber_sequence.incremented
"""


RETURN = r"""
incremented:
  description:
    - It is set to V(true) if the attribute value has changed.
  returned: success
  type: bool
  sample: true

attribute:
  description:
    - The name of the attribute that was incremented.
  returned: success
  type: str
  sample: uidNumber

value:
  description:
    - The new value after incrementing.
  returned: success
  type: str
  sample: "2"

rfc4525:
  description:
    - Is V(true) if the method used to increment is based on RFC4525, V(false) if legacy.
  returned: success
  type: bool
  sample: true
"""

from ansible.module_utils.basic import AnsibleModule, missing_required_lib
from ansible.module_utils.common.text.converters import to_bytes

from ansible_collections.community.general.plugins.module_utils import _deps as deps
from ansible_collections.community.general.plugins.module_utils._ldap import (
    LdapGeneric,
    gen_specs,
    ldap_required_together,
)

with deps.declare("ldap", reason=missing_required_lib("python-ldap")):
    import ldap
    import ldap.controls.readentry


class LdapInc(LdapGeneric):
    def __init__(self, module):
        LdapGeneric.__init__(self, module)
        # Shortcuts
        self.attr = self.module.params["attribute"]
        self.increment = self.module.params["increment"]
        self.method = self.module.params["method"]

    def inc_rfc4525(self):
        return [(ldap.MOD_INCREMENT, self.attr, [to_bytes(str(self.increment))])]

    def inc_legacy(self, curr_val, new_val):
        return [(ldap.MOD_DELETE, self.attr, [to_bytes(curr_val)]), (ldap.MOD_ADD, self.attr, [to_bytes(new_val)])]

    def serverControls(self):
        return [ldap.controls.readentry.PostReadControl(attrList=[self.attr])]

    LDAP_MOD_INCREMENT = to_bytes("1.3.6.1.1.14")


def main():
    module = AnsibleModule(
        argument_spec=gen_specs(
            attribute=dict(type="str", required=True),
            increment=dict(type="int", default=1),
            method=dict(type="str", default="auto", choices=["auto", "rfc4525", "legacy"]),
        ),
        supports_check_mode=True,
        required_together=ldap_required_together(),
    )

    deps.validate(module)

    # Instantiate the LdapAttr object
    mod = LdapInc(module)

    changed = False
    ret = ""
    rfc4525 = False

    try:
        if mod.increment != 0 and not module.check_mode:
            changed = True

            if mod.method != "auto":
                rfc4525 = mod.method == "rfc425"
            else:
                rootDSE = mod.connection.search_ext_s(base="", scope=ldap.SCOPE_BASE, attrlist=["*", "+"])
                if len(rootDSE) == 1:
                    if to_bytes(ldap.CONTROL_POST_READ) in rootDSE[0][1]["supportedControl"] and (
                        mod.LDAP_MOD_INCREMENT in rootDSE[0][1]["supportedFeatures"]
                        or mod.LDAP_MOD_INCREMENT in rootDSE[0][1]["supportedExtension"]
                    ):
                        rfc4525 = True

            if rfc4525:
                dummy, dummy, dummy, resp_ctrls = mod.connection.modify_ext_s(
                    dn=mod.dn, modlist=mod.inc_rfc4525(), serverctrls=mod.serverControls(), clientctrls=None
                )
                if len(resp_ctrls) == 1:
                    ret = resp_ctrls[0].entry[mod.attr][0]

            else:
                tries = 0
                max_tries = 3
                while tries < max_tries:
                    tries = tries + 1
                    result = mod.connection.search_ext_s(
                        base=mod.dn, scope=ldap.SCOPE_BASE, filterstr=f"({mod.attr}=*)", attrlist=[mod.attr]
                    )
                    if len(result) != 1:
                        module.fail_json(msg="The entry does not exist or does not contain the specified attribute.")
                        return
                    try:
                        ret = str(int(result[0][1][mod.attr][0]) + mod.increment)
                        # if the current value first arg in inc_legacy has changed then the modify will fail
                        mod.connection.modify_s(dn=mod.dn, modlist=mod.inc_legacy(result[0][1][mod.attr][0], ret))
                        break
                    except ldap.NO_SUCH_ATTRIBUTE:
                        if tries == max_tries:
                            module.fail_json(msg=f"The increment could not be applied after {max_tries} tries.")
                            return

        else:
            result = mod.connection.search_ext_s(
                base=mod.dn, scope=ldap.SCOPE_BASE, filterstr=f"({mod.attr}=*)", attrlist=[mod.attr]
            )
            if len(result) == 1:
                ret = str(int(result[0][1][mod.attr][0]) + mod.increment)
                changed = mod.increment != 0
            else:
                module.fail_json(msg="The entry does not exist or does not contain the specified attribute.")

    except Exception as e:
        module.fail_json(msg="Attribute action failed.", details=f"{e}")

    module.exit_json(changed=changed, incremented=changed, attribute=mod.attr, value=ret, rfc4525=rfc4525)


if __name__ == "__main__":
    main()



# Copyright (c) 2015-2021, Felix Fontein <felix@fontein.de>
# Copyright (c) 2018 Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later
from __future__ import annotations

DOCUMENTATION = r"""
name: dependent
short_description: Composes a list with nested elements of other lists or dicts which can depend on previous loop variables
author: Felix Fontein (@felixfontein)
version_added: 3.1.0
description:
  - Takes the input lists and returns a list with elements that are lists, dictionaries, or template expressions which evaluate
    to lists or dicts, composed of the elements of the input evaluated lists and dictionaries.
options:
  _terms:
    description:
      - A list where the elements are one-element dictionaries, mapping a name to a string, list, or dictionary. The name
        is the index that is used in the result object. The value is iterated over as described below.
      - If the value is a list, it is simply iterated over.
      - If the value is a dictionary, it is iterated over and returned as if they would be processed by the P(ansible.builtin.dict2items#filter)
        filter.
      - If the value is a string, it is evaluated as Jinja2 expressions which can access the previously chosen elements with
        C(item.<index_name>). The result must be a list or a dictionary.
    type: list
    elements: dict
    required: true
"""

EXAMPLES = r"""
- name: Install/remove public keys for active admin users
  ansible.posix.authorized_key:
    user: "{{ item.admin.key }}"
    key: "{{ lookup('file', item.key.public_key) }}"
    state: "{{ 'present' if item.key.active else 'absent' }}"
  when: item.admin.value.active
  with_community.general.dependent:
    - admin: admin_user_data
    - key: admin_ssh_keys[item.admin.key]
  loop_control:
    # Makes the output readable, so that it doesn't contain the whole subdictionaries and lists
    label: "{{ [item.admin.key, 'active' if item.key.active else 'inactive', item.key.public_key] }}"
  vars:
    admin_user_data:
      admin1:
        name: Alice
        active: true
      admin2:
        name: Bob
        active: true
    admin_ssh_keys:
      admin1:
        - private_key: keys/private_key_admin1.pem
          public_key: keys/private_key_admin1.pub
          active: true
      admin2:
        - private_key: keys/private_key_admin2.pem
          public_key: keys/private_key_admin2.pub
          active: true
        - private_key: keys/private_key_admin2-old.pem
          public_key: keys/private_key_admin2-old.pub
          active: false

- name: Update DNS records
  community.aws.route53:
    zone: "{{ item.zone.key }}"
    record: "{{ item.prefix.key ~ '.' if item.prefix.key else '' }}{{ item.zone.key }}"
    type: "{{ item.entry.key }}"
    ttl: "{{ item.entry.value.ttl | default(3600) }}"
    value: "{{ item.entry.value.value }}"
    state: "{{ 'absent' if (item.entry.value.absent | default(False)) else 'present' }}"
    overwrite: true
  loop_control:
    # Makes the output readable, so that it doesn't contain the whole subdictionaries and lists
    label: |-
      {{ [item.zone.key, item.prefix.key, item.entry.key,
          item.entry.value.ttl | default(3600),
          item.entry.value.absent | default(False), item.entry.value.value] }}
  with_community.general.dependent:
    - zone: dns_setup
    - prefix: item.zone.value
    - entry: item.prefix.value
  vars:
    dns_setup:
      example.com:
        '':
          A:
            value:
              - 1.2.3.4
          AAAA:
            value:
              - "2a01:1:2:3::1"
        'test._domainkey':
          TXT:
            ttl: 300
            value:
              - '"k=rsa; t=s; p=MIGfMA..."'
      example.org:
        'www':
          A:
            value:
              - 1.2.3.4
              - 5.6.7.8
"""

RETURN = r"""
_list:
  description:
    - A list composed of dictionaries whose keys are the variable names from the input list.
  type: list
  elements: dict
  sample:
    - key1: a
      key2: test
    - key1: a
      key2: foo
    - key1: b
      key2: bar
"""

from collections.abc import Mapping, Sequence

from ansible.errors import AnsibleLookupError
from ansible.plugins.lookup import LookupBase
from ansible.template import Templar

from ansible_collections.community.general.plugins.plugin_utils._lookup import check_for_wrong_terms

try:
    from ansible.template import trust_as_template as _trust_as_template

    HAS_DATATAGGING = True
except ImportError:
    HAS_DATATAGGING = False


def _make_safe(value):
    if HAS_DATATAGGING and isinstance(value, str):
        return _trust_as_template(value)
    return value


class LookupModule(LookupBase):
    def __evaluate(self, expression, templar, variables):
        """Evaluate expression with templar.

        ``expression`` is the expression to evaluate.
        ``variables`` are the variables to use.
        """
        templar.available_variables = variables or {}
        open_br = "{{"
        close_br = "}}"
        quoted_expression = f"{open_br}{expression}{close_br}"
        if hasattr(templar, "evaluate_expression"):
            # This is available since the Data Tagging PR has been merged
            return templar.evaluate_expression(_make_safe(expression))
        return templar.template(quoted_expression)

    def __process(self, result, terms, index, current, templar, variables):
        """Fills ``result`` list with evaluated items.

        ``result`` is a list where the resulting items are placed.
        ``terms`` is the parsed list of terms
        ``index`` is the current index to be processed in the list.
        ``current`` is a dictionary where the first ``index`` values are filled in.
        ``variables`` are the variables currently available.
        """
        # If we are done, add to result list:
        if index == len(terms):
            result.append(current.copy())
            return

        key, expression, values = terms[index]

        if expression is not None:
            # Evaluate expression in current context
            vars = variables.copy()
            vars["item"] = current.copy()
            try:
                values = self.__evaluate(expression, templar, variables=vars)
            except Exception as e:
                raise AnsibleLookupError(f'Caught "{e}" while evaluating {key!r} with item == {current!r}') from e

        if isinstance(values, Mapping):
            for idx, val in sorted(values.items()):
                current[key] = dict(key=idx, value=val)
                self.__process(result, terms, index + 1, current, templar, variables)
        elif isinstance(values, Sequence):
            for elt in values:
                current[key] = elt
                self.__process(result, terms, index + 1, current, templar, variables)
        else:
            raise AnsibleLookupError(
                f"Did not obtain dictionary or list while evaluating {key!r} with item == {current!r}, but {type(values)}"
            )

    def run(self, terms, variables=None, **kwargs):
        """Generate list."""
        self.set_options(var_options=variables, direct=kwargs)
        check_for_wrong_terms(self, direct=kwargs)

        result = []
        if len(terms) > 0:
            if HAS_DATATAGGING:
                templar = self._templar.copy_with_new_env(available_variables={})
            else:
                templar = Templar(loader=self._templar._loader)
            data = []
            vars_so_far = set()
            for index, term in enumerate(terms):
                if not isinstance(term, Mapping):
                    raise AnsibleLookupError(f"Parameter {index} must be a dictionary, got {type(term)}")
                if len(term) != 1:
                    raise AnsibleLookupError(
                        f"Parameter {index} must be a one-element dictionary, got {len(term)} elements"
                    )
                k, v = list(term.items())[0]
                if k in vars_so_far:
                    raise AnsibleLookupError(f"The variable {k!r} appears more than once")
                vars_so_far.add(k)
                if isinstance(v, str):
                    data.append((k, v, None))
                elif isinstance(v, (Sequence, Mapping)):
                    data.append((k, None, v))
                else:
                    raise AnsibleLookupError(
                        f"Parameter {k!r} (index {index}) must have a value of type string, dictionary or list, got type {type(v)}"
                    )
            self.__process(result, data, 0, {}, templar, variables)
        return result



# Copyright (c) 2022 Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this plugin util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

# NOTE: THIS IS ONLY FOR FILTER PLUGINS!

from __future__ import annotations

import typing as t

from ansible.errors import AnsibleFilterError
from ansible.utils.display import Display

_display = Display()


class FilterModuleMock:
    def __init__(self, params: dict[str, t.Any]) -> None:
        self.check_mode = True
        self.params = params
        self._diff = False

    def fail_json(self, msg: str, **kwargs: t.Any) -> t.NoReturn:
        raise AnsibleFilterError(msg)

    def warn(self, warning: str) -> None:
        _display.warning(warning)


__all__ = ("FilterModuleMock",)



# Copyright (c) 2025 Ansible project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

"""
Module utils for cryptography requirements.

Must be kept in sync with plugins/doc_fragments/cryptography_dep.py.
"""

from __future__ import annotations

import traceback
import typing as t

from ansible.module_utils.basic import missing_required_lib

from ansible_collections.community.crypto.plugins.module_utils._version import (
    LooseVersion,
)

if t.TYPE_CHECKING:
    from ansible.module_utils.basic import AnsibleModule  # pragma: no cover

    from ansible_collections.community.crypto.plugins.plugin_utils._action_module import (  # pragma: no cover
        AnsibleActionModule,
    )
    from ansible_collections.community.crypto.plugins.plugin_utils._filter_module import (  # pragma: no cover
        FilterModuleMock,
    )

    GeneralAnsibleModule = t.Union[  # noqa: UP007
        AnsibleModule, AnsibleActionModule, FilterModuleMock
    ]  # pragma: no cover


_CRYPTOGRAPHY_IMP_ERR: str | None = None  # pylint: disable=invalid-name
_CRYPTOGRAPHY_FILE: str | None = None  # pylint: disable=invalid-name
try:
    import cryptography
    from cryptography import x509  # noqa: F401, pylint: disable=unused-import

except ImportError:
    _CRYPTOGRAPHY_IMP_ERR = traceback.format_exc()  # pylint: disable=invalid-name
    CRYPTOGRAPHY_FOUND = False
    CRYPTOGRAPHY_VERSION = LooseVersion("0.0")  # pylint: disable=invalid-name
else:
    CRYPTOGRAPHY_FOUND = True
    # pylint: disable-next=invalid-name
    CRYPTOGRAPHY_VERSION = LooseVersion(cryptography.__version__)
    _CRYPTOGRAPHY_FILE = cryptography.__file__  # pylint: disable=invalid-name


# Corresponds to the community.crypto.cryptography_dep.minimum doc fragment
COLLECTION_MINIMUM_CRYPTOGRAPHY_VERSION = "3.3"


def assert_required_cryptography_version(
    module: GeneralAnsibleModule,
    *,
    minimum_cryptography_version: str = COLLECTION_MINIMUM_CRYPTOGRAPHY_VERSION,
) -> None:
    if not CRYPTOGRAPHY_FOUND:
        module.fail_json(
            msg=missing_required_lib(f"cryptography >= {minimum_cryptography_version}"),
            exception=_CRYPTOGRAPHY_IMP_ERR,
        )
    if LooseVersion(minimum_cryptography_version) > CRYPTOGRAPHY_VERSION:
        module.fail_json(
            msg=(
                f"Cannot detect the required Python library cryptography (>= {minimum_cryptography_version})."
                f" Only found a too old version ({CRYPTOGRAPHY_VERSION}) at {_CRYPTOGRAPHY_FILE}."
            ),
        )


__all__ = (
    "COLLECTION_MINIMUM_CRYPTOGRAPHY_VERSION",
    "CRYPTOGRAPHY_FOUND",
    "CRYPTOGRAPHY_VERSION",
    "assert_required_cryptography_version",
)



# Copyright (c) 2018, Ansible Project
# Simplified BSD License (see LICENSES/BSD-2-Clause.txt or https://opensource.org/licenses/BSD-2-Clause)
# SPDX-License-Identifier: BSD-2-Clause

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import traceback
import typing as t

from ansible.module_utils.basic import env_fallback, missing_required_lib

if t.TYPE_CHECKING:
    from ansible.module_utils.basic import AnsibleModule

HAS_HEROKU = False
HEROKU_IMP_ERR = None
try:
    import heroku3

    HAS_HEROKU = True
except ImportError:
    HEROKU_IMP_ERR = traceback.format_exc()


class HerokuHelper:
    def __init__(self, module: AnsibleModule) -> None:
        self.module = module
        self.check_lib()
        self.api_key = module.params["api_key"]

    def check_lib(self) -> None:
        if not HAS_HEROKU:
            self.module.fail_json(msg=missing_required_lib("heroku3"), exception=HEROKU_IMP_ERR)

    @staticmethod
    def heroku_argument_spec() -> dict[str, t.Any]:
        return dict(
            api_key=dict(fallback=(env_fallback, ["HEROKU_API_KEY", "TF_VAR_HEROKU_API_KEY"]), type="str", no_log=True)
        )

    def get_heroku_client(self):
        client = heroku3.from_key(self.api_key)

        if not client.is_authenticated:
            self.module.fail_json(msg="Heroku authentication failure, please check your API Key")

        return client
